package keenGame;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
import java.util.AbstractCollection;
import java.util.Collection;
import java.util.Iterator;
import java.util.NoSuchElementException;

import keenGame.sprite.MovementEvent;
import keenGame.sprite.MovementListener;
import keenGame.sprite.Sprite;

/**
 * Array Map, is a collection of ArrayMapLayers.  It is used to create
 * a layered type level, ie. one where objects can be behind
 * or infront of each other.  This means that sprites will be returned
 * by iterators, begining at the lowest layer, and moving up to the
 * heighest layer.  This is usefull for rendering, and also collision
 * detection, where sprites only collide with objects in certian layers.
 * 
 * An ArrayMap must listen to the moving objects, that it stores, so that
 * it may update their position
 * 
 * Each layer is represented by a string.  You specify which layers the
 * ArrayMap contains in the constructor, and they canot be changed once
 * the ArrayMap is instiantated.
 * 
 * Sprites are added into the layer returned by their getLayer() method.  If
 * the specifed layer does not exist an UnsupportedOperationException will occur
 */
public class ArrayMap extends AbstractCollection implements MovementListener
{
	private ArrayMapLayer [] maps;
	private String [] layers;
	private int width;
	private int height;
	

	/**
	 * Constructor for ArrayMap.  This will create a ArrayMap with the
	 * specified layers, and all other details will be taken from the
	 * defaults provided by ArrayMapLayer()
	 * 
	 * @param layers - An array of strings, representing the layers to
	 * be added into the ArrayMap.  Later layers my be referenced by
	 * their string representation.  layers[0] is the lowest layer, and
	 * the sprites it contains will be returned first by an iterator
	 */
	public ArrayMap(String [] layers)
	{
		this.layers = layers;
		maps = new ArrayMapLayer [layers.length];
		
		for (int i = 0; i < layers.length; i++)
		{
			maps[i] = new ArrayMapLayer();
		}
		this.width = maps[0].getWidth();
		this.height = maps[0].getHeight();
	}
	
	/**
	 * Constructor for ArrayMap.  This will create a ArrayMap with the
	 * specified layers, width and height and all other details will be 
	 * taken from the defaults provided by ArrayMapLayer()
	 * 
	 * @param layers - An array of strings, representing the layers to
	 * be added into the ArrayMap.  Later layers my be referenced by
	 * their string representation.  layers[0] is the lowest layer, and
	 * the sprites it contains will be returned first by an iterator
	 * 
	 * @param width - The minimum width of the map
	 * @param height - The height of the map
	 */
	public ArrayMap(int width, int height, String [] layers)
	{
		this.layers = layers;
		maps = new ArrayMapLayer [layers.length];
		
		for (int i = 0; i < layers.length; i++)
		{
			maps[i] = new ArrayMapLayer(width, height);
		}
		this.width = maps[0].getWidth();
		this.height = maps[0].getHeight();
	}

	/**
	 * Constructor for ArrayMap.  This will create a ArrayMap with the
	 * specified layers, width, height, cellSize and maximum 
	 * number of sprites per cell.
	 * 
	 * @param layers - An array of strings, representing the layers to
	 * be added into the ArrayMap.  Later layers my be referenced by
	 * their string representation.  layers[0] is the lowest layer, and
	 * the sprites it contains will be returned first by an iterator
	 * @param width - The minimum width of the map
	 * @param height - The height of the map
	 * @param cellSize - The size of each cell.  Cells are square
	 * @param maxSprites - The maximum number of sprites per cell
	 * 
	 */
	public ArrayMap(int width, int height, String [] layers, int cellSize, int maxSprites)
	{
		this.layers = layers;
		maps = new ArrayMapLayer [layers.length];
		
		for (int i = 0; i < layers.length; i++)
		{
			maps[i] = new ArrayMapLayer(width, height ,cellSize, maxSprites);
		}
		this.width = maps[0].getWidth();
		this.height = maps[0].getHeight();
	}

	/**
	 * Given a string representing a layer, finds index of the
	 * specified layer in the maps array.  Throws an UnsupportedOperationException
	 * if the layer cannot be found.  Presently it conducts a linear search, because
	 * there are so few layers, however it could easily be modified to work on a hash
	 * table.
	 * 
	 * @param toFind - A string representing the layer to be found
	 * @return the index of the layer in the maps array
	 * @throws An UnsupportedOperationException if the layer cannot be found.
	 */
	private int findLayer(String toFind)
	{
		int layer = 0;
		try
		{
			while(!layers[layer].equals(toFind))
			{
				layer++;
			}
		}
		catch(ArrayIndexOutOfBoundsException e)
		{
			throw new UnsupportedOperationException("Layer " + toFind + " not found");
		}
		return layer;
	}	

	/**
	 * Adds the provided sprite to the collection.  This method will throw
	 * a ClassCastException if the paramiter canot be cast to a sprite.  The
	 * x, y, width and height and layer of the sprite will be taken from the 
	 * sprite its self
	 * 
	 * @param newSprite - The sprite to be added to the collection
	 * @throws UnsupportedOperationException if the Sprite has dimensions that
	 * cannot be accomidated inside the map, or if there are to many sprites in 
	 * one of the cells that the sprite wishes to occupy
	 */
	public boolean add(Object newSprite)
	{
		maps[findLayer(((Sprite)newSprite).getLayer())].add(newSprite);		
		return true;
	}
	
	/**
	 * Removes the provided sprite to the collection.  This method will throw
	 * a ClassCastException if the paramiter canot be cast to a sprite.  The
	 * x, y, width and height and layer of the sprite will be taken from the
	 * sprite its self
	 * 
	 * @param removeSprite - The sprite to be removed from the collection
	 * @return false is the specified sprite could not be found
	 */	
	public boolean remove(Object toRemove)
	{
		maps[findLayer(((Sprite)toRemove).getLayer())].remove(toRemove);		
		return true;
	}
	
	/**
	 * Adds in a new layers, above all the other layers.
	 * 
	 * @param newLayer - The name of the new layer to be added
	 */
	public void addLayer(String newLayer)
	{
		String [] newLayers = new String[layers.length + 1];
		ArrayMapLayer [] newMaps = new ArrayMapLayer[maps.length + 1];
		
		//copy old layers, and maps
		for(int i = 0; i < layers.length; i++)
		{
			newLayers[i] = layers[i];
			newMaps[i] = maps[i];
		}
		
		newLayers[newLayers.length - 1] = newLayer;
		newMaps[newMaps.length - 1] = new ArrayMapLayer(width,height)		;
		
		layers = newLayers;
		maps = newMaps;
	}
	
	/**
	 * Removes the layer specified.
	 * 
	 * @param removeLayer - The name of the layer to be removed
	 */
	public void removeLayer(String removeLayer)
	{
		String [] newLayers = new String[layers.length - 1];
		ArrayMapLayer [] newMaps = new ArrayMapLayer[maps.length - 1];
		
		//copy old layers, and maps
		int numLayers = 0;
		for(int i = 0; i < layers.length; i++)
		{
			if(!(layers[i].equals(removeLayer)))
			{
				newLayers[numLayers] = layers[i];
				newMaps[numLayers] = maps[i];
				numLayers++;
			}
		}
		
		layers = newLayers;
		maps = newMaps;
	}
	
	/**
	 * Changes the name of the specified layer
	 * 
	 * @param currentName - The name of the layer which we wish to change
	 * @param newName - The new name for the layer
	 */
	public void renameLayer(String currentName, String newName)
	{
		//create new layer
		ArrayMapLayer newLayer = new ArrayMapLayer(width, height);
		
		//get iterator from old layer
		int location = findLayer(currentName);
		Iterator it = maps[location].iterator();
		
		while(it.hasNext())
		{
			Sprite currentSprite = (Sprite)it.next();
			
			//rename layer in sprite
			currentSprite.setLayer(newName);

			//add to new layer
			newLayer.add(currentSprite);
		}
		
		//add new layer into maps
		maps[location] = newLayer;
		//add new name into layers
		layers[location] = newName;
	}
	
	/**
	 * Swaps the ordering of two specified layers
	 * 
	 * @param layer1 - the first layer to be swaped
	 * @param layer2 - the second layer to be swaped
	 */
	public void swapLayers(String layer1, String layer2)
	{
		int index1 = findLayer(layer1);
		int index2 = findLayer(layer2);
		
		//copy layer 1 to temp
		ArrayMapLayer tempLayer = maps[index1];
		
		//copy layer 2 to 1
		maps[index1] = maps[index2];
		layers[index1] = layer2;
		
		//copy temp to 2
		maps[index2] = tempLayer;
		layers[index2] = layer1;
	}
	
	/**
	 * Removes all sprites stored in the ArrayMap
	 */
	public void clear()
	{
		for (int i = 0; i < maps.length; i++)
		{
			maps[i].clear();
		}
	}
	
	/**
	 * Returns the height of the ArrayMap
	 * @return int
	 */	
	public int getHeight()
	{
		return height;
	}

	/**
	 * Returns the width of the ArrayMap
	 * @return int
	 */			
	public int getWidth()
	{
		return width;
	}
	
	/**
	 * Returns an array containing all names of the layers
	 * contained by the map.  The layers will be returned in
	 * the same order they were added by the contructor.  ie
	 * lowest layer is [0], etc, etc
	 */			
	public String []  getLayers()
	{
		return layers;
	}
	
	/**
	 * Returns the number of sprites stored inside the collection
	 */
	public int size()
	{
		//Sum up size
		int sum = 0;
		for (int i = 0; i < maps.length; i++)
		{
			sum = sum + maps[i].size();
		}
		return sum;
	}

	public boolean removeAll(Collection toRemove)
	{
		boolean success = true;
		
		Iterator it = toRemove.iterator();
		while(it.hasNext())
		{
			if(!remove(it.next()))
			{
				success = false;
			}
		}
		
		return success;
	}

	/**
	 * Returns an iterator that will run from the top corner of the lowest 
	 * layer in the map, to the bottom right of the highest layer in the
	 * map and will return each sprite stored in the map only once
	 * 
	 * @return A default iterator that will iterate through the whole map
	 */
	public Iterator iterator()
	{
		return new ArrayMapIterator(this, getLayers());
	}

	/**
	 * Returns an iterator that will run from the top corner of the specifed 
	 * layer in the map, to the bottom right of the specified layer in the
	 * map and will return each sprite stored in the map only once
	 * 
	 * @return A default iterator that will iterate through the whole map
	 */
	public Iterator iterator(String layer)
	{
		String [] array = {layer};
		return new ArrayMapIterator(this, array);
	}

	/**
	 * Returns an iterator that will run from the top corner of the specifed 
	 * layers in the map, to the bottom right of the specified layers in the
	 * map and will return each sprite stored in the map only once
	 * 
	 * @return A default iterator that will iterate through the whole map
	 */
	public Iterator iterator(String [] layers)
	{
		return new ArrayMapIterator(this, layers);
	}

	/**
	 * Returns an iterator that will run through all the sprites contained
	 * inside the specifed box.  It will return each sprite only once.  The
	 * iterator will return sprites from all layers
	 */	
	public Iterator iterator(int x, int y, int width, int height)
	{
		return new ArrayMapIterator(this, getLayers(), x, y, width, height);
	}

	/**
	 * Returns an iterator that will run through all the sprites contained
	 * inside the specifed box.  It will return each sprite only once.  The
	 * iterator will return sprites from only the specifed layer
	 */	
	public Iterator iterator(int x, int y, int width, int height, String layer)
	{
		String [] array = {layer};
		return new ArrayMapIterator(this, array, x, y, width, height);
		
	}

	/**
	 * Returns an iterator that will run through all the sprites contained
	 * inside the specifed box.  It will return each sprite only once.  The
	 * iterator will return sprites from only the specifed layers
	 */	
	public Iterator iterator(int x, int y, int width, int height, String [] layers)
	{
		return new ArrayMapIterator(this, layers, x, y, width, height);
	}
	
	/**
	 * Used by movementPerformed, to remove the sprite from its old position
	 */
	private boolean removeFrom(int x, int y, Object toRemove)
	{
		return maps[findLayer(((Sprite)toRemove).getLayer())].removeFrom(x, y,
				((Sprite)toRemove).getWidth(),
				((Sprite)toRemove).getHeight(),
				 toRemove);		
	}
	
	/**
	 * Adjusts the position of a Sprite inside the ArrayMap
	 */
	public void movementPerformed(MovementEvent e)
	{
		removeFrom(e.getOldX(), e.getOldY(), e.getSource());
		add(e.getSource());
	}
	
	/**
	 * Array Map iterator will return all sprites inside a specified area, and
	 * layers, in order from the sprites in the lowest layer, to the highest layer.
	 * You can chose which area and/or layers to iterate through.  Each sprite
	 * will be returned only once
	 * 
	 * @see ArrayMapLayer#SpriteIterator
	 */
	private class ArrayMapIterator implements Iterator
	{
		private Iterator innerIt;
		private Object next;
		private boolean hasNext = true;
		private String [] layers;
		private String currentLayerName;
		private int currentLayerNumber;
		private ArrayMap iterateMap;
		
		private int startX, startY, width, height;

		/**
		 * Creats an iterator that will iterate through evey element in the
		 * entire ArrayMap, inside the specified layers
		 * 
		 * @param iterateMap - The Array map through which to iterate
		 */
		private ArrayMapIterator(ArrayMap iterateMap, String [] layers)
		{
			currentLayerName = layers[0];
			this.layers = layers;
			startX = 0;
			startY = 0;
			width = iterateMap.getWidth();
			height = iterateMap.getHeight();
			this.iterateMap = iterateMap;
			
			//Create an iterator at the first layer
			innerIt = maps[iterateMap.findLayer(layers[currentLayerNumber])].iterator(startX, startY, width, height);
			findNext();
		}
		
		/**
		 * Creates an iterator that will iterate through evey element in the
		 * specifed box ArrayMap, inside the specified area, and layers
		 * 
		 * @param iterateMap - The Array map through which to iterate
		 * @param x - The left of the box
		 * @param y - The top of the box
		 * @param width - The width of the box, through which to iterate
		 * @param height - the height of the box through which to iterate
		 */
		private ArrayMapIterator(ArrayMap iterateMap, String [] layers, int x, int y, int width, int height)
		{
			currentLayerName = layers[0];
			currentLayerNumber = 0;
			this.layers = layers;
			startX = x;
			startY = y;
			this.width = width;
			this.height = height;
			this.iterateMap = iterateMap;
			
			//Create an iterator at the first level
			innerIt = maps[iterateMap.findLayer(layers[currentLayerNumber])].iterator(startX, startY, width, height);
			findNext();
		}
		
		/**
		 * Finds the next valid sprite.  This method makes calls to innerIt untill
		 * it finds a new sprite.  If innerIt.hasNext() == false then it will
		 * create a new iterator, for the next layer.
		 */		private void findNext()
		{
			//If this iterator has more
			if(innerIt.hasNext())
			{
				//get Next
				next = innerIt.next();
			}
			else if(currentLayerName.equals(layers[layers.length - 1]))
			//if this layer is the last layer
			{
				//we have no more
				hasNext = false;
				next = null;
			}
			else //We should move onto the next layer
			{
				//Get iterator from the next layer
				currentLayerNumber++;
				currentLayerName = iterateMap.layers[iterateMap.findLayer(layers[currentLayerNumber])];

				innerIt = maps[iterateMap.findLayer(layers[currentLayerNumber])].iterator(startX, startY, width, height);
				findNext();
			}
		}
		
		public boolean hasNext()
		{
			return hasNext;
		}
		
		public Object next()
		{
			Object toReturn = next;
			if(hasNext)
			{
				findNext();
				return toReturn;
			}
			else
			{
				throw new NoSuchElementException("Iterator has no more elements");
			}
		}
		
		/**
		 * @throws UnsupportedOperationException
		 */
		public void remove()
		{
			throw new UnsupportedOperationException("Remove not supported for this type of iterator");
		}
	}
}
