package keenGame;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.util.AbstractCollection;
import java.util.Iterator;
import java.util.NoSuchElementException;

import keenGame.sprite.Sprite;

/**
 * ArrayMapLayer is a single layer of cells, used for storing, and looking up objects by X and Y value.
 * A CellMapLayer is usefull in for creating 2D game, because it allows all sprites apoximately inside a box, to be
 * looked up with a small big O of 1.  This is usefull for rendering, or collision detection, where we need
 * fast look up of all the sprites in an area.
 * 
 * A ArrayMapLayer is an N x N matrix of "Cells" which can be of any size, and contain a number of spirtes
 * You can set the size of each cell, as well as the maximum number of sprites each cell can contain via the
 * ArrayMapLayer(int, int, int) constructor.  It is possible to make a make an ArrayMapLayer consisting of 1x1 cells, which
 * can contain only one sprite, it is better to create an ArrayMapLayer with larger cells, that can contain more than
 * one sprite.  This allows savings on memory.  However if you create a cell map with cells that are two large, 
 * you will need to allow each cell, to contain to many sprites, and it will slow lookup time.  
 */
public class ArrayMapLayer extends AbstractCollection
{
	private static final int DEFAULT_WIDTH = 2000;
	private static final int DEFAULT_HEIGHT = 2000;
	
	private static final int DEFAULT_CELL_SIZE = 20;
	private static final int DEFAULT_MAX_SPRITES = 20; //The number of sprites that you can have ontop 
										 //of each other at any one time.
										 
	private int height;
	private int width;
	private int cellSize;
	private int maxSprites;
	
	private int size; //Keeps the number of sprites currently in the ArrayMapLayer
	
	
	private Sprite [][][] map;
	
	private static int numIterations;
	public static int getNumIterations()
	{
		int toReturn = ArrayMapLayer.numIterations;
		ArrayMapLayer.numIterations = 0;
		return toReturn;
	}


	/**
	 * Constructor for CellMapLayer.
	 * Provides a default, width, height, cell size, and max sprites.  It is
	 * recomended that you at least use the ArrayMapLayer(int, int) constructor
	 */
	public ArrayMapLayer()
	{
		this(DEFAULT_WIDTH, DEFAULT_HEIGHT, DEFAULT_CELL_SIZE, DEFAULT_MAX_SPRITES);
	}
	
	/**
	 * Creates an ArrayMapLayer with the specified width and height.  This means
	 * you cannot add anythign that would exceed the bounds of the map.  This
	 * constructor provides default values for cell size, and the maximum number
	 * of sprites.  Note that the width and height are taken as minimum, values.
	 * Due to rounding, the map may actually be larger.
	 * 
	 * @param width - The minimum width of the map
	 * @param height - The height of the map
	 */
	public ArrayMapLayer(int width, int height)
	{
		this(width, height, DEFAULT_CELL_SIZE, DEFAULT_MAX_SPRITES);
	}
	/**
	 * Creates an ArrayMapLayer with the specified width, height, cell size, and
	 * maximum number of sprites per cell.  This means you cannot add anythign 
	 * that would exceed the bounds of the map.Note that the width and height are taken as minimum, values.
	 * Due to rounding, the map may actually be larger.
	 * 
	 * @param width - The minimum width of the map
	 * @param height - The height of the map
	 * @param cellSize - The size of each cell.  Cells are square
	 * @param maxSprites - The maximum number of sprites per cell
	 */
	public ArrayMapLayer(int width, int height, int cellSize, int maxSprites)
	{
		this.width = width;
		this.height = height;
		this.cellSize = cellSize;
		this.maxSprites = maxSprites;
		
		
		int mapWidth = (int)Math.ceil(((double)width)/((double)cellSize));
		int mapHeight = (int)Math.ceil(((double)height)/((double)cellSize));
		
		map = new Sprite [mapWidth][mapHeight][maxSprites];
	}

	/**
	 * Returns an iterator that will run from the top corner of the map, 
	 * to the bottom right, and will return each sprite stored in the
	 * map only once
	 * @return A default iterator that will iterate through the whole map
	 */
	public Iterator iterator()
	{
		return new SpriteIterator(this);
	}
	
	/**
	 * Returns an iterator that will run through all the sprites contained
	 * inside the specifed box.  It will return each sprite only once
	 */
	public Iterator iterator(int x, int y, int width, int height)
	{
		return new SpriteIterator(this, x, y, width, height);
	}
	
	/**
	 * Adds the provided sprite to the collection.  This method will throw
	 * a ClassCastException if the paramiter canot be cast to a sprite.  The
	 * x, y, width and height of the sprite will be taken from the sprite its
	 * self
	 * 
	 * @param newSprite - The sprite to be added to the collection
	 * @throws UnsupportedOperationException if the Sprite has dimensions that
	 * cannot be accomidated inside the map, or if there are to many sprites in 
	 * one of the cells that the sprite wishes to occupy
	 */
	public boolean add(Object newSprite)
	{
		size++;
		
		//convert sprite co-ords to map co-ords
		int spriteX = ((Sprite)newSprite).getLocation().x;
		int spriteY = ((Sprite)newSprite).getLocation().y;
		int spriteFinishX = spriteX + ((Sprite)newSprite).getWidth();
		int spriteFinishY = spriteY + ((Sprite)newSprite).getHeight();
		
		int mapX = (int)Math.floor((double)spriteX / (double)cellSize);
		int mapY = (int)Math.floor((double)spriteY / (double)cellSize);
		
		//We use ceil, because we want to finish the loop < instead of <=
		int mapFinishX = (int)Math.ceil((double)spriteFinishX / (double)cellSize);
		int mapFinishY = (int)Math.ceil((double)spriteFinishY / (double)cellSize);
		
		//Check that it fits
		if((mapX < 0) || (mapY < 0) || (mapFinishX > map.length) || (mapFinishY > map[0].length))
		{
			throw new UnsupportedOperationException("Sprite does not fit inside ArrayMapLayer");
		}
		
		//iterate around
		for(int x = mapX; x < mapFinishX; x++)
		{
			for(int y = mapY; y < mapFinishY; y++)
			{
				int depth;
				
				//find the first null
				for(depth = 0; (depth < maxSprites) && (map[x][y][depth] != null); depth++)
				{}
				
				if((depth == (maxSprites-1)) && (map[x][y][depth] != null))
				{
					//then we are out of space in this cell
					throw new UnsupportedOperationException("Cell " + x + "," + y + " has to many sprites");
				}
				else //We found a null, add it in
				{
					map[x][y][depth] = (Sprite)newSprite;
				}
			}
		}
		return true;
	}

	
	/**
	 * Removes all sprites stored in the ArrayMapLayer
	 */
	public void clear()
	{
		size = 0;
		
		//Loop around and null out eveything
		for(int x = 0; x < map.length; x++)
		{
			for(int y = 0; y < map[0].length; y++)
			{
				for(int depth = 0; depth < map[0][0].length; depth++)
				{
					map[x][y][depth] = null;
				}
			}
		}
	}
	
	/**
	 * Removes the provided sprite to the collection.  This method will throw
	 * a ClassCastException if the paramiter canot be cast to a sprite.  The
	 * x, y, width and height of the sprite will be taken from the sprite its
	 * self
	 * 
	 * @param removeSprite - The sprite to be removed from the collection
	 * @return false is the specified sprite could not be found
	 */
	public boolean remove(Object removeSprite)
	{
		return removeFrom(((Sprite)removeSprite).getX(),
							((Sprite)removeSprite).getY(), 
							((Sprite)removeSprite).getWidth(),
							((Sprite)removeSprite).getHeight(),
							removeSprite);
	}
	
	/**
	 * Attempts to remove the provided sprite to the collection at the given
	 * co-ordinates.  This method will throw a ClassCastException if the 
	 * paramiter canot be cast to a sprite.
	 * 
	 * @param removeSprite - The sprite to be removed from the collection
	 * @param removeX - The X positions from which to begin removing
	 * @param removeY - The Y positions from which to begin removing
	 * @param width - The width of the area to remove
	 * @param height - The height of the area to remove
	 * 
	 * @return false is the specified sprite could not be found
	 */
	public boolean removeFrom(int removeX, int removeY, int width, int height, Object removeSprite)
	{
		boolean removed = false;

		size--;
		//convert sprite co-ords to map co-ords
		int spriteX = removeX;
		int spriteY = removeY;
		int spriteFinishX = spriteX + width;
		int spriteFinishY = spriteY + height;
		
		int mapX = (int)Math.floor((double)spriteX / (double)cellSize);
		int mapY = (int)Math.floor((double)spriteY / (double)cellSize);
		
		//We use ceil, because we want to finish the loop < instead of <=
		int mapFinishX = (int)Math.ceil((double)spriteFinishX / (double)cellSize);
		int mapFinishY = (int)Math.ceil((double)spriteFinishY / (double)cellSize);
		
		//iterate around
		for(int x = mapX; x < mapFinishX; x++)
		{
			for(int y = mapY; y < mapFinishY; y++)
			{
				int depth;
				
				//find the sprite
				boolean found = false;
				for(depth = 0; (depth < maxSprites) && !found; depth++)
				{
					if(map[x][y][depth] == removeSprite)
					{
						map[x][y][depth] = null;
						removed = true;
						found = true;
						//System.err.println("removed");
					}
				}
			}
		}
		return removed;
	}

	/**
	 * Returns the number of sprites stored in the ArrayMapLayer
	 * @see java.util.Collection#size()
	 */
	public int size()
	{
		return size;
	}
	
	/**
	 * Returns the height of the ArrayMapLayer.
	 * @return int
	 */
	public int getHeight()
	{
		return height;
	}

	/**
	 * Returns the width of the ArrayMapLayer
	 * @return int
	 */
	public int getWidth()
	{
		return width;
	}

	
	/**
	 * An iterator that will iterate through evey element in a 
	 * specified Box, excluding null elements
	 */
	private class CellIterator implements Iterator
	{
		private int x, y, depth;
		private int startX;
		private int endX, endY;
		private ArrayMapLayer iterateMap;
		
		/**
		 * Creates an iterator that will iterate through evey element in the
		 * entire ArrayMapLayer
		 * 
		 * @param iterateMap - The Array map through which to iterate
		 */
		private CellIterator(ArrayMapLayer iterateMap)
		{
			this(iterateMap, 0, 0, iterateMap.getWidth(), iterateMap.getHeight());
		}
		
		/**
		 * Creates an iterator that will iterate through evey element in the
		 * specifed box ArrayMapLayer
		 * 
		 * @param iterateMap - The Array map through which to iterate
		 * @param x - The left of the box
		 * @param y - The top of the box
		 * @param width - The width of the box, through which to iterate
		 * @param height - the height of the box through which to iterate
		 */
		private CellIterator(ArrayMapLayer iterateMap, int startX, int startY, int width, int height)
		{
			x = (int)Math.floor((double)startX / (double)iterateMap.cellSize);
			this.startX = x;
			y = (int)Math.floor((double)startY / (double)iterateMap.cellSize);
		
			endX = (int)Math.ceil((double)(startX + width) / (double)cellSize);
			endX--;
			endY = (int)Math.ceil((double)(startY + height) / (double)cellSize);
			endY--;
			
			depth = 0;
			
			this.iterateMap = iterateMap;
		}
				
		public boolean hasNext()
		{
			return !((x == endX) &&
					(y == endY) &&
					(depth == (iterateMap.map[0][0].length)- 1));
		}
		
		public Object next() //20% of CPU time is spent in here
		{
			ArrayMapLayer.numIterations++;
			Object toReturn = iterateMap.map[x][y][depth++];

			if(depth == iterateMap.map[0][0].length)
			{
				depth = 0;
				x++;

				if(x > endX)
				{
					x = startX;
					y++;
				}
			}

			//To save from making to many calls to next() we need to filter out any null
			while((toReturn == null) && 
					(!((x == endX) && //Has next
					(y == endY) &&
					(depth == (iterateMap.map[0][0].length)- 1))))
			{
				toReturn = iterateMap.map[x][y][depth++];
				
				if(depth == iterateMap.map[0][0].length)
				{
					depth = 0;
					x++;
	
					if(x > endX)
					{
						x = startX;
						y++;
					}
				}
			}
			
			return toReturn;
		}
		
		/**
		 * @throws UnsupportedOperationException
		 */
		public void remove()
		{
			throw new UnsupportedOperationException("Remove not supported for this type of iterator");
		}
	}
	
	/**
	 * An iterator that will iterate through evey sprite in a 
	 * specified Box, excluding null elements.  Each element in the
	 * box will be returned only once
	 */
	private class SpriteIterator implements Iterator
	{
		private CellIterator innerIt;
		private Object next;
		private boolean hasNext;

		private int startX, startY;
		
		private Object previous; //used for remove
		
		
		/**
		 * Creats an iterator that will iterate through evey element in the
		 * entire ArrayMapLayer
		 * 
		 * @param iterateMap - The Array map through which to iterate
		 */
		private SpriteIterator(ArrayMapLayer iterateMap)
		{
			innerIt = new CellIterator(iterateMap);
			startX = innerIt.x;
			startY = innerIt.y;
			findNext();
		}
		
		/**
		 * Creates an iterator that will iterate through evey element in the
		 * specifed box ArrayMapLayer
		 * 
		 * @param iterateMap - The Array map through which to iterate
		 * @param x - The left of the box
		 * @param y - The top of the box
		 * @param width - The width of the box, through which to iterate
		 * @param height - the height of the box through which to iterate
		 */
		private SpriteIterator(ArrayMapLayer iterateMap, int x, int y, int width, int height)
		{
			innerIt = new CellIterator(iterateMap, x, y, width, height);
			startX = innerIt.x;
			startY = innerIt.y;
			findNext();
			
		}

		/**
		 * Finds the next valid sprite.  This method makes calls to innerIt untill
		 * it finds a new sprite, or innerIt.hasNext() == false
		 */
		private void findNext() //25% of CPU time is spent in here
		{
			if(innerIt.hasNext())
			{
				next = innerIt.next();
			}
			else
			{
				next = null;
			}
				
			while((!isUpperLeft((Sprite)next, innerIt.x, innerIt.y)) &&
					(innerIt.hasNext()))
			{
				next = innerIt.next();
			}
			hasNext = innerIt.hasNext();
		}
		
		private boolean isUpperLeft(Sprite checkSprite, int x, int y)
		{
			//Figure out if we have returned it before it before
			if(checkSprite == null)
			{
				return false;
			}
			
			//Calculate the origin
			int spriteX = checkSprite.getLocation().x;
			int spriteY = checkSprite.getLocation().y;
		
			int originX = (int)Math.floor((double)spriteX / (double)cellSize);
			int originY = (int)Math.floor((double)spriteY / (double)cellSize);
			
			//Check to see if it is to the left or above of where we started iterating
			if(originX < startX)
			{
				originX = startX;
			}
			if(originY < startY)
			{
				originY = startY;
			}
			
			//Are we at its upper left corner
			if((originX == x) && (originY == y))
			{
				return true;
			}
			else
			{
				return false;
			}
		}
		
		public boolean hasNext()
		{
			return hasNext;
		}
		
		public Object next()
		{
			Object toReturn = next;
			if(hasNext)
			{
				findNext();
				previous = toReturn;
				return toReturn;
			}
			else
			{
				throw new NoSuchElementException("Iterator has no more elements");
			}
		}
		
		public void remove()
		{
			System.err.println("removing " + previous);
			innerIt.iterateMap.remove(previous);
		}
	}
}
