package keenGame;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.awt.AWTException;
import java.awt.BufferCapabilities;
import java.awt.CardLayout;
import java.awt.Cursor;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsEnvironment;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.image.MemoryImageSource;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

import javax.swing.JFrame;

import keenGame.sprite.keenActor.KeenActor;


/**
 * Keen 9 Application.  Handels all menu's and starts up levels
 * Places its self in full screen mode, using class ScreenManager
 * this means it is safe to exit the program at any time, without
 * worrying about screen resolution.
 *
 * Keen is also responsible for loading up levels, and hideing its
 * self while they run.  It provides action events to an actor
 * which is passes to the currently running level
 *
 * @version 0.0, April 25th 2002
 * @author Luke Sleeman
 */

public class Keen extends JFrame 
{
	private static final String DEFAULT_LEVEL = "levels/testLevel.xml";
	
	private Screen mainMenu = new MainMenuScreen(this);
	private ConfigMenuScreen config = new ConfigMenuScreen(this);
	private Level levelOne;
	private Screen loading = new LoadingScreen(this);
	private CardLayout screens;

	//Commander keen!
	private KeenActor keyActor;
	private boolean configAdded = false;

    private static int BUFFERS = 2;
	
	public final static int DEFAULT_WIDTH = 640;
	public final static int DEFAULT_HEIGHT = 400;
	
	
    /**
     * Constructs Keen, displaying the intro screen, and the menu
     */
    public Keen()
    {
  		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setSize(DEFAULT_WIDTH,DEFAULT_HEIGHT);
		screens = new CardLayout();
		getContentPane().setLayout(screens);
		
		getContentPane().add(mainMenu, "mainMenu");
		getContentPane().add(config, "config");
		getContentPane().add(loading, "loading");

		screens.show(getContentPane(),"mainMenu");
    }
    
    public void screenFinished(Screen doneScreen, int message)
    {
    	if(doneScreen == mainMenu)
    	{
    		if(message == MainMenuScreen.RUN_GAME)
    		{
    			screens.show(getContentPane(),"loading");
    			loading.setVisible(true);
				update(getGraphics());
    			
    			createBufferStrategy(BUFFERS);
    			//get the buffer capabilities

    			BufferCapabilities bufferCap = getBufferStrategy().getCapabilities();
    			//set its flip strategy
    			try
    			{
					createBufferStrategy(BUFFERS, new BufferCapabilities(bufferCap.getFrontBufferCapabilities(),
										bufferCap.getBackBufferCapabilities(), BufferCapabilities.FlipContents.COPIED));
    			}
    			catch(AWTException e)
    			{
    				System.err.println(e);
    				System.exit(0);
    			}
    			
				GraphicsConfiguration gConf = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration();
				Cache.setGraphicsConfig(gConf);
				Cache.setClassLoader(getClass().getClassLoader());
				
				//ACK!  Hides the mouse cursor
				setCursor(Toolkit.getDefaultToolkit().createCustomCursor(createImage(new MemoryImageSource(256, 256, new int[1], 0, 256)),new Point(0, 0), "empty"));
				getContentPane().setVisible(false);
				
				InputStream file;
				if(((ConfigMenuScreen)config).getFile() != null)
				{
					try
					{
						file = new FileInputStream(((ConfigMenuScreen)config).getFile());
					}
					catch(FileNotFoundException e)
					{
						System.err.println(e);
						file = Cache.getClassLoader().getResourceAsStream(DEFAULT_LEVEL);
					}
				}
				else
				{
					file = Cache.getClassLoader().getResourceAsStream(DEFAULT_LEVEL);
				}

				try
				{
					levelOne = new Level(file,  
										getBufferStrategy(),
										 this,
										config.getMovement(),config.getDebug());
				}
				catch(IOException e)
				{
					System.err.println("Error trying to load up data");
					e.printStackTrace();
					System.exit(0);
				}
				
				//levelOne.setIgnoreRepaint(true);
				//config.setIgnoreRepaint(true);
				//mainMenu.setIgnoreRepaint(true);
				//setIgnoreRepaint(true);

				//TODO find a way around the swing focus problems
				//Make sure we have the focus    			
				requestFocus();
				addKeyListener(levelOne);
				addKeyListener(levelOne.getPlayer());

				//Start the level running									
				new Thread(levelOne).start(); 
    		}
    		else if(message == MainMenuScreen.EXIT)
    		{
    			System.exit(0);
    		}
    		else if(message == MainMenuScreen.CONFIG)
    		{
    			screens.show(getContentPane(),"config");
	   			config.requestFocus();
    		}
    	}
    	else if(doneScreen == config)
    	{
   			screens.show(getContentPane(),"mainMenu");
   			mainMenu.requestFocus();
    	}
    	else if(doneScreen == levelOne)
    	{
			getContentPane().setVisible(true);
			setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
   			screens.show(getContentPane(),"mainMenu");
   			repaint();
   			mainMenu.requestFocus();
    	}
    }
    
      
    /**
     * Main method, creates an instance of Keen, and displays it in
     * full screen
     */
    public static void main(String args [])
    {

        //Set full screen
        Keen game;
        game = new Keen();
        game.setVisible(true);
    }
}
      
