package keenGame;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.image.BufferStrategy;
import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import keenGame.sprite.keenActor.KeenActor;
import keenGame.sprite.keenActor.KeenActorMovement;
import keenGame.sprite.Actor;
import keenGame.sprite.MovingActor;
import keenGame.sprite.Sprite;

/**
 * Class Level represents a level in the game.  It is actually a
 * runnable thread.  Level is responsible for loading up all its
 * data as well, as running the main game loop.
 *
 * @version 0.0, April 25th 2002
 * @author Luke Sleeman
 */

public class Level extends Screen implements Runnable, KeyListener
{
	private boolean run = true;

	private static final long RENDER_INTERVAL = 30; //Our ideal for how long we should wait between frames
							//Setting it to low will starve the other threads, thus keyboard input
							//will be very laggy
							//Setting it to high will give a low frame rate
	private static final long MIN_SLEEP_TIME = 10;
	
	//Stuff is working

	private static final int NUMBER_OF_OBJECTS = 500; //Should be limited around 500, otherwise FPS will suffer
	private static final int MAX_OVERLAYS = 1;
	private static final int FINISHED_DISPLAY_TIME = 1000;
	
	//private static final double SCALE = 1.1;
	
	//Used for scheduler    
	private long nextRenderTime;
	private long sleepTime;

	private long stopTime = 0; //How long untill we stop
	private Image endImage;
    
	//Actors
	private Actor [] Actors;
	private int numActors = 0;
	private KeenActor keenPlayer;
    
	private ArrayMap positionMap; //BlockMap
	private ViewPort window; //ViewPort
	private BufferStrategy screen; //Graphics
    
	private Overlay [] overlays = new Overlay[MAX_OVERLAYS];
	private int numOverlays = 0;
	private debugOverlay debug;
	private boolean renderDebug = true;
	
	private RepaintManager dirtyArea = new RepaintManager();

	private int oldViewX;
	private int oldViewY;
    
	/**
	* Creates a new level loading data into it
	*
	* @param newFileName - File name of the level to be loaded
	* @param keyActor - An Actor which will handel keyboard events
	* @param graphConfig - The graphics configuration the level 
	*                         will be run on (used for sprites to create themselves into)
	* @param bufferStrategy - The screen the level will display its self on
	*/
	public Level(InputStream file, BufferStrategy screen, Keen app, KeenActorMovement movement, boolean renderDebug) throws IOException
	{
		super(app);
		this.screen = screen;
		this.renderDebug = renderDebug;

		//Initilise level data
		LevelLoader levelData = new XMLLevelLoader2(file, this);

		Actors = new Actor[NUMBER_OF_OBJECTS];

		positionMap = new ArrayMap(levelData.getWidth(), 
							levelData.getHeight(),
							levelData.getLayers());

		//Add in keen							
		keenPlayer = levelData.getKeen();
		keenPlayer.setMovement(movement);  
		addSprite(keenPlayer);

		//Viewport
		window = new ViewPort(keenPlayer, 
								levelData.getWidth(), levelData.getHeight(), 
								getViewportWidth(), getViewportHeight());
								//(int)(((double)getViewportWidth()) / SCALE), (int)(((double)getViewportHeight()) / SCALE));


		//System.out.println((int)(((double)getViewportWidth()) / SCALE) + ", " +(int)(((double)getViewportHeight()) / SCALE));


		//Make sure we render the full level this time
		dirtyArea.addRegion(window.getX(), window.getY(), window.getWidth(), window.getHeight());

		
		//Add in all the rest
		Iterator it = levelData.getSprites().iterator();
		while (it.hasNext())
		{
			addSprite((Sprite)it.next());
		}
		
		//create our overlays
		debug = new debugOverlay();
		overlays[0] = debug;
		numOverlays++; 
		
		//set up the end image
		endImage = Cache.getScaledImage(Cache.getImage("graphics/endLevel.gif"), 3.0);
	}
    
	public Level(int width, int height, String [] layers)
	{
    	super(null);
    	
    	Actors = new Actor[NUMBER_OF_OBJECTS];

		positionMap = new ArrayMap(width, 
							height,
							layers);
	}
    
	/**
	* Begins the level thread running.  This contains the main loop, which will
	* periodically update all the actors in the level, and render the screen.
	* Eventhing inside this loop, should be highly optimised. The following are
	* pointers to the behavour inside the loop:
	* * No new objects should be created, othwise the garbage collector will
	*   induce alot of lag
	* * No advanced collections should be used, as this will add in to much overhead
	* * The number of method calls should be minimised
	*
	* Remember :- framerate is GOD!
	*/
	public void run()
	{
		boolean stopLooping = false;
		run = true;
		Graphics g;

		try
		{
			//We should try and initlise ALL our values here, outside the loop
			//nextFPStime = System.currentTimeMillis() + 1000;
	        
			//Garbage collect so we are not doing it in the loop
			System.gc(); 
			System.runFinalization();
			System.gc(); 
			nextRenderTime = System.currentTimeMillis();
        
			while(!stopLooping) //Here is our main loop! --------------------------------------------------------------------- God lives here ;-)
			{
				// Render a frame 
				g = screen.getDrawGraphics();
	
				//Update the level
				update();
	            
				//Render level
				renderLevel(g, window.getX(), window.getY(), window.getWidth(), window.getHeight());
	            
				//Render any overlays
				if(renderDebug)
				{
					renderOverlays(g, window.getWidth(), window.getHeight());
				}
	
				//if we are should be stopping
				if(!run)
				{
					drawFinished(g);
					stopLooping = System.currentTimeMillis() > stopTime;
				}
	
				//Flip it in!
				g.dispose();
				screen.show();
	                        
				//If we have got any spare time, sleep so the other threads can go
				//LoopTimer.blockUntillTime();
				sleepTime = nextRenderTime - System.currentTimeMillis();
				debug.setSleepTime(sleepTime);
				if(sleepTime > MIN_SLEEP_TIME)
				{
					Thread.sleep(sleepTime);
					debug.setSleepTime(sleepTime);
				}
				else
				{
					Thread.sleep(MIN_SLEEP_TIME);
					debug.setSleepTime(MIN_SLEEP_TIME);
				}
				nextRenderTime = nextRenderTime + RENDER_INTERVAL;

				if(nextRenderTime < System.currentTimeMillis())
				{
					nextRenderTime = System.currentTimeMillis();
				}
			} // Optmize the hell out of anything in here ------------------------------------------------------------
		}
		catch(Exception e)
		{
			System.err.println ("Error in main loop!");
			System.err.println ();
			e.printStackTrace();
			System.exit(0);
		}

		//Once the game loop is finished, return to the main menu
		done(0);
	}
    
	private void drawFinished(Graphics g)
	{
		//draw finished
		int imageX = (window.getWidth() / 2) - (endImage.getWidth(null) / 2);
		int imageY = (window.getHeight() / 2) - (endImage.getHeight(null) / 2);
		
		g.setClip(imageX, imageY, endImage.getWidth(null), endImage.getHeight(null));
		g.drawImage(endImage, imageX, imageY, null);
	}
    
	/**
	* loops though all the Actors in the world, and updates them if
	* neccessary.  It then renders the the current viewport of the world
	* to the provided graphics object.  
	*/
	public void update()
	{
		int tempNumActors = 0;
		oldViewX = window.getX();
		oldViewY = window.getY();
		//Update all actors
		
		for(int i = 0;i < numActors; i++)
		{
        	
		if(Actors[i].timeForUpdate())
		{
			int oldX = Actors[i].getX();
			int oldY = Actors[i].getY();
			int oldX2 = oldX + Actors[i].getWidth();
			int oldY2 = oldY + Actors[i].getHeight();
			
			Actors[i].update(); //Update the object
			Actors[i].setFrame(); //Set to the correct frame in an animation
			
			int repaintX, repaintY, repaintWidth, repaintHeight;
			
			int newX = Actors[i].getX();
			int newY = Actors[i].getY();
			int newX2 = newX + Actors[i].getWidth();
			int newY2 = newY + Actors[i].getHeight();
			
			if(oldX < newX)
			{
				repaintX = oldX;
			}
			else
			{
				repaintX = newX;
			}

			if(oldY < newY)
			{
				repaintY = oldY;
			}
			else
			{
				repaintY = newY;
			}
			
			if(oldX2 > newX2)
			{
				repaintWidth = oldX2 - repaintX;
			}
			else
			{
				repaintWidth = newX2 - repaintX;
			}

			if(oldY2 > newY2)
			{
				repaintHeight = oldY2 - repaintY;
			}
			else
			{
				repaintHeight = newY2 - repaintY;
			}

			dirtyArea.addRegion(repaintX, repaintY, repaintWidth, repaintHeight);
			//dirtyArea.addRegion(Actors[i].getX(), Actors[i].getY(), Actors[i].getWidth(), Actors[i].getHeight());
		}
            
		//compress the new array
		if(Actors[i].isEnabled())
		{
			if(tempNumActors != i)
			{
				Actors[tempNumActors] = Actors[i];
				Actors[i] = null;
			}
				tempNumActors++;
			}
			else //its been removed, so we have to take it out of the positionMap
			{
				positionMap.remove(Actors[i]);
			}
		}
		numActors = tempNumActors;
	}

	public void renderLevel(Graphics g, int x, int y, int width, int height)
	{
		renderLevel(g, x, y, width, height, false, getLayers());
	}

	public void renderLevel(Graphics g, int x, int y, int width, int height, boolean debug, String [] layers)
	{
		//Blank it
		g.setColor(new Color(88,87,104));

		if(x < 0)
		{
			x = 0;
		}
		if(y < 0)
		{
			y =0;
		}
        
		if((x + width) > positionMap.getWidth())
		{
			width = positionMap.getWidth() - x;
		}
		if((y + height) > positionMap.getHeight())
		{
			height = positionMap.getHeight() - y;
		}
             
		//Tell the positionMap to render the current viewport on the screen
		int numObjects = 0;

		if(!debug) //use fast rendering
		{
			if((oldViewX != window.getX()) || (oldViewY != window.getY())) //If we have had some scrolling
			{
				//figure out delta
				int deltaX =  oldViewX - window.getX();
				int deltaY =  oldViewY - window.getY();

				//set clip
				//g.setClip(0,0,(int)(window.getWidth() * SCALE), (int)(window.getHeight() * SCALE));
				g.setClip(0,0,window.getWidth(), window.getHeight());


				//copy area
				g.copyArea(0,0,window.getWidth(), window.getHeight(),
							deltaX, deltaY);
				//g.copyArea(0,0,(int)(window.getWidth() *SCALE), (int)(window.getHeight() *SCALE),
				//			(int)(deltaX * SCALE * 1.12), (int)(deltaY * SCALE * 1.12));


				//add undrawn regions to area to redraw
				//if right, add left area
				if(deltaX > 0) //we need to add in the area to the left
				{
					dirtyArea.addRegion(window.getX(), window.getY(),
										deltaX, window.getHeight());
				}
				else if(deltaX < 0) //we need to add in the area to the right
				{
					dirtyArea.addRegion((window.getX() + window.getWidth() ) + deltaX , window.getY(),
										-deltaX, window.getHeight());
				}

				if(deltaY > 0) //we need to add in the area to the top
				{
					dirtyArea.addRegion(window.getX(), window.getY(),
										window.getWidth(), deltaY);
				}
				else if(deltaY < 0) //we need to add in the area to the bottom
				{
					dirtyArea.addRegion(window.getX() , (window.getY() + window.getHeight()) + deltaY,
										window.getWidth(),-deltaY);
				}

				//redraw the regions that were under any overlays
				for (int i = 0; i < numOverlays; i++)
				{
					dirtyArea.addRegion((overlays[i].getX(width) + deltaX) + x, 
										(overlays[i].getY(height) + deltaY) + y,
										overlays[i].getWidth(width),
										overlays[i].getHeight(height));
				}
				
				
				//dirtyArea.addRegion(window.getX(), window.getY(), window.getWidth(), window.getHeight());
			}


			//Get list of area
			Rectangle[] dirty = dirtyArea.getDirtyRegions(x,y,width, height);

			for (int i = 0; i < dirty.length; i++)
			{
				//set clip
				//g.setClip((int)((dirty[i].x  - x ) * SCALE),
				//		(int)((dirty[i].y  - y ) * SCALE),
				//		(int)(dirty[i].width * SCALE),
				//		(int)(dirty[i].height * SCALE));
				g.setClip(dirty[i].x  - x ,
						dirty[i].y - y ,
						dirty[i].width,
						dirty[i].height);

				//get iterator for area
				Iterator it = positionMap.iterator(dirty[i].x, dirty[i].y, dirty[i].width, dirty[i].height, layers);
				while(it.hasNext())
				{
					((Sprite)it.next()).draw(g,x ,y);
					//((Sprite)it.next()).draw(g,x ,y , false, 1.0);
					//((Sprite)it.next()).draw(g, (int)((double)x * SCALE), (int)((double)y * SCALE) , true, SCALE);

					numObjects++;
				}
			}
		}
		else //Use slow rendering
		{        
			Iterator it = positionMap.iterator(x, y, width, height, layers);
			while(it.hasNext())
			{
				((Sprite)it.next()).draw(g, x, y);
				//((Sprite)it.next()).draw(g, (int)((double)x * SCALE), (int)((double)y * SCALE) , false, SCALE);
				//((Sprite)it.next()).draw(g, x, y);

				numObjects++;
			}
		}
		dirtyArea.clearRegions();

		this.debug.setObjects(numObjects);
	}
	
	private void renderOverlays(Graphics g, int width, int height)
	{
		for (int i = 0; i < numOverlays; i++)
		{
			overlays[i].render(g, width, height);
		}
		ArrayMapLayer.getNumIterations();
	}
    
	public void stop()
	{
		run = false;
		stopTime = System.currentTimeMillis() + FINISHED_DISPLAY_TIME;
	}
    
	public void scrollUp()
	{
		window.moveUp();
	}
    
	public void scrollDown()
	{
		window.moveDown();
	}

    
	public void keyPressed(KeyEvent e)
	{
		if(e.getKeyCode() == KeyEvent.VK_ESCAPE)
		{
				if(run == true)
				{
					stop();
				}        	
		}
	}
    
	public void keyReleased(KeyEvent e)
	{}

	public void keyTyped(KeyEvent e)
	{}
    
	/**
	 * Adds an actor into the world
	 *
	 * WARNING:  This method, is dangerious, in that it may incure GC lag.
	 * new objects should not be created inside the loop.  
	 *
	 * TODO: re-implement, to use a chache of pre-created objects
	 */
	public void addSprite(Sprite newSprite)
	{
		//check if it is a moving actor
		//try
		if(newSprite instanceof MovingActor)
		{
			((MovingActor)newSprite).addMovementListener(positionMap);
		}
	
		if(newSprite instanceof Actor)
		{
			if(numActors < NUMBER_OF_OBJECTS)
			{
				Actors[numActors] = (Actor)newSprite;
				numActors++;
			}
		}
		positionMap.add(newSprite);
	}
    
	/**
	 * Changes the layer the specified sprite is in.
	 * 
	 * @param toChange - The sprite whoes layer we will change
	 * @param newLayer - The layer we which to place the sprite in
	 */
	public void changeLayer(Sprite toChange, String newLayer)
	{
		//remove sprites
		positionMap.remove(toChange);
			
		toChange.setLayer(newLayer);
		positionMap.add(toChange);
	}
    
	/**
	 * Ugly, but we need this to get around swings focus problems
	 */
	public KeenActor getPlayer()
	{
		return keenPlayer;
	}

	public int getWidth()
	{
		return positionMap.getWidth();
	}

	public int getHeight()
	{
		return positionMap.getHeight();
	}

	public String [] getLayers()
	{
		return positionMap.getLayers();
	}

	public Iterator iterator()
	{
		return positionMap.iterator();
	}
    
	public Iterator iterator(int x, int y, int width, int height, String layer)
	{
		return positionMap.iterator(x,y,width, height, layer);
	}

	public Iterator iterator(int x, int y, int width, int height, String [] layers)
	{
		return positionMap.iterator(x,y,width, height, layers);
	}
}
