package keenGame;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import keenGame.sprite.MovementEvent;
import keenGame.sprite.MovementListener;
import keenGame.sprite.MovingActor;

/**
 * Models a viewport into a level.  A ViewPort can follow an
 * object, by listening to its movement.  The viewport will 
 * attempt to keep the object it is following visibile.  A 
 * viewport can also be controled from outside, however it will
 * always try and keep the object it is following visible.
 * 
 */
public class ViewPort implements MovementListener
{
	private final static int MOVE_DISTANCE = 4;
	
	/**
	 * The percentage of the width of the ViewPort for which
	 * movments of the actor will not cause the viewport to scroll.
	 * eg, a value of .5, means that the actor can move freely without
	 * causing a scroll
	 */
	private final static double SCROLL_WIDTH = .5;
	
	private int x, y, width, height;
	private int levelWidth, levelHeight;
	
	private MovingActor toFollow;
	/**
	 * Creates a viewport, to follow the specified MovingActor.
	 * 
	 * @param toFollow - the movingActor the viewport will follow
	 * @param levelWidth - the width of the world, within which the viewport
	 * 						must operate
	 * @param levelHeight - the height of the world, within which the viewport
	 * 						must operate
	 * @param viewportWidth - the width of the viewport
	 * @param viewportHeight - the height of the viewport
	 */
	public ViewPort(MovingActor toFollow, int levelWidth, int levelHeight, 
					int viewportWidth, int viewportHeight)
	{
		//Add listener to actor
		toFollow.addMovementListener(this);
		this.toFollow = toFollow;
		this.levelWidth = levelWidth;
		this.levelHeight = levelHeight;
		width = viewportWidth;
		height = viewportHeight;
		
		//Create centered on sprite
		x = toFollow.getX() + (toFollow.getWidth() / 2);
		x = x - (width / 2);
		
		y = toFollow.getY() + (toFollow.getHeight() / 2);
		y = y - (height / 2);
		
		validate();
	}

	/**
	 * Returns the X location of the upper left of the viewport
	 */
	public int getX()
	{
		return x;
	}

	/**
	 * Returns the Y location of the upper left of the viewport
	 */
	public int getY()
	{
		return y;
	}

	/**
	 * Returns 	the width of the viewport
	 */
	public int getWidth()
	{
		return width;
	}
	/**
	 * Returns 	the height of the viewport
	 */
	public int getHeight()
	{
		return height;
	}
	
	/**
	 * Recomends that the viewport scroll upwards.  This will only
	 * succeed, if the viewport can remain within the bounds of the
	 * level, and the MovingActor the viewport is tracking will stil
	 * be visible
	 */
	public void moveUp()
	{
		y = y - MOVE_DISTANCE;
		validate();
	}

	/**
	 * Recomends that the viewport scroll downwards.  This will only
	 * succeed, if the viewport can remain within the bounds of the
	 * level, and the MovingActor the viewport is tracking will stil
	 * be visible
	 */
	public void moveDown()
	{
		y = y + MOVE_DISTANCE;
		validate();
	}
	
	/**
	 * Ensures that all rules for the viewport are met.  eg, the actor
	 * we are following is visible, and the viewport is within the confines
	 * of the level
	 */
	private void validate()
	{
		//ensure actor is not above viewport
		if(toFollow.getY() < (y + (height * SCROLL_WIDTH / 2) ))
		{
			//revalidate
			y = toFollow.getY() - (int)(height * SCROLL_WIDTH / 2);
		}

		//ensure actor is not below viewport
		if((toFollow.getY() + toFollow.getHeight()) > 
			((y + height) - (height * SCROLL_WIDTH / 2) ))
		{
			//revalidate
			y = height - (toFollow.getHeight() + (int)(height * SCROLL_WIDTH / 2));
			y = toFollow.getY() - y;
		}

		//ensure actor is not to the left viewport
		if(toFollow.getX() < (x + (width * SCROLL_WIDTH / 2) ))
		{
			//revalidate
			x = toFollow.getX() - (int)(width * SCROLL_WIDTH / 2);
		}

		//ensure actor is not to the right viewport
		if((toFollow.getX() + toFollow.getWidth()) > 
			((x + width) - (width * SCROLL_WIDTH / 2) ))
		{
			//revalidate
			x = width - (toFollow.getWidth() + (int)(width * SCROLL_WIDTH / 2));
			x = toFollow.getX() - x;
		}
		
		//Adjust for being above the top left
		if(x < 0)
		{
			x = 0;
		}
		if(y < 0)
		{
			y = 0;
		}
		
		//Adjust for being below the bottom right
		if(x > (levelWidth - width))
		{
			x = levelWidth - width;
		}
		if(y > (levelHeight - height))
		{
			y = levelHeight - height;
		}
	}
	
	/**
	 * Updates the position of the viewport, to track the actor it is following
	 * @see keenGame.sprite.MovementListener#movementPerformed(MovementEvent)
	 */
	public void movementPerformed(MovementEvent e)
	{
		//Adjust so keen is still within viewport
		if(e.getSource() == toFollow)
		{
			validate();
		}
	}

}
