package keenGame;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.util.Vector;

import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

/**
 * An XmlElement wraps an org.w3c.dom.Element providing usefull functionality
 * allowing it to be easily used, as well as hiding useless functinality.  
 * XmlElements provide all the functionality needed for simple loading
 * and parsing of an Xml file, however lack advanced functionality.  If you need
 * this, it may be better to use org.w3c.dom.Element directly
 */
public class XmlElement
{
	
	private Element theElement;
	/**
	 * Constructs an XmlElement, wraping the specified Element
	 * 
	 * @param wrapElement - The element, that this XmlElement shal wrap
	 */
	public XmlElement(Element wrapElement)
	{
		theElement = wrapElement;
	}
	
	/**
	 * Returns the Element wrapped by the XmlElement
	 */
	public Element getElement()
	{
		return theElement;
	}
	
	/**
	 * Returns the name of the element.  Eg for:
	 * <pre>
	 * 	<person>joe</person>
	 * </pre>
	 * you will get back "person"
	 * 
	 * @return The name of the element
	 */
	public String getName()
	{
		return theElement.getTagName();
	}
	
	/**
	 * Returns the vale of the specified attribute.  If it does
	 * not exist a null will be returned
	 * 
	 * @param name - The name of the attribute to get
	 * @return The value of the attribute
	 */
	public String getAttribute(String name)
	{
		String attribute = theElement.getAttribute(name);
		
		if(attribute.equals(""))
		{
			return null;
		}
		else
		{
			return attribute;
		}
	}
	
	/**
	 * Returns a list of the attributes contained in the element
	 * @return An array of strings, representing the names of attributes
	 * 			contained in the element
	 */
	public String [] getAttributes()
	{
		NamedNodeMap attrNodes = theElement.getAttributes();
		String [] attributes = new String [attrNodes.getLength()];
		
		for(int i = 0; i < attributes.length; i++)
		{
			attributes[i] = attrNodes.item(i).getNodeName();
		}
		return attributes;
	}
	
	/**
	 * Gets the first child node of the XmlElement, with the specified name.
	 * If no children with the specified name exist, null will be returned
	 * @return A child Element, wrapped in an XmlElement
	 */
	public XmlElement getChild(String name)
	{
		NodeList list = theElement.getElementsByTagName(name);
		
		XmlElement toReturn;
		
		if(list.getLength() > 1)
		{
			throw new RuntimeException("This element has more than one child named " + name);
		}
		else if(list.getLength() == 0)
		{
			toReturn = null;
		}
		else
		{
			toReturn = new XmlElement((Element)list.item(0));
		}
		
		return toReturn;
	}
	
	/**
	 * Returns all children of the element
	 */
	public XmlElement [] getChildren()
	{
		//get list of children
		NodeList childrenNodes = theElement.getChildNodes();
		Vector children = new Vector();

		for(int i = 0; i < childrenNodes.getLength(); i++)
		{
			Node child = childrenNodes.item(i);
			if(child instanceof Element)
			{
				children.add(new XmlElement((Element)child));
			}
		}
		
		if(children.size() > 0)
		{
			return (XmlElement[]) children.toArray(new XmlElement[children.size()]);
		}
		else
		{
			return new XmlElement [0];
		}
	}

	/**
	 * Gets the text contained by the element
	 * @return The text contained by the element
	 */
	public String getText()
	{
		StringBuffer tempString = new StringBuffer();
		NodeList children = theElement.getChildNodes();
		for(int i = 0; i < children.getLength(); i++)
		{
			Node child = children.item(i);
			if (child instanceof Text)
			{
 				tempString.append( child.getNodeValue() );
			}
		}
		return tempString.toString();
	}
	
	/**
	 * Returns the text contained by the element, as a integer
	 * @return The integer contained by the element
	 */
	public int getValue()
	{
		return Integer.parseInt(getText());
	}
	
	public String toString()
	{
		return theElement.toString();
	}
}
