package keenGame.editor;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.JPanel;
import javax.swing.JScrollPane;

import keenGame.ArrayMap;
import keenGame.Cache;
import keenGame.LevelLoader;
import keenGame.XMLLevelLoader2;
import keenGame.editor.tools.*;
import keenGame.sprite.Sprite;
import keenGame.sprite.keenActor.KeenActor;

public class LevelPanel extends JPanel
{
	private static final Color BACKGROUND = new Color(88,87,104);
	private static final Color GRID_COLOR = Color.ORANGE;
	private static final Color SPRITE_BOX_COLOR = Color.BLUE;
	private static final Color SELECTION_BOX_COLOR = Color.RED;
	public static final int DEFAULT_LEVEL_WIDTH = 2000;
	public static final int DEFAULT_LEVEL_HEIGHT = 2000;
	
	private ArrayMap theLevel;
	private JPanel drawPanel;
	
	private boolean gridVisible = false;
	private boolean snapToGrid = false;
	private int gridSize = 20;
	private double scale = 1.0;

	private boolean spriteBoxVisible;
	private Rectangle spriteBox = new Rectangle();
	
	private String [] visibleLayers;

	private boolean selectedSpritesVisible;
	private Vector selecedSprites = new Vector();
	
	private KeenActor keen;
	
	/**
	 * Constructor for LevelPanel.
	 */
	public LevelPanel()
	{
		super();
		
		setLayout(new BorderLayout());
		
		//Set up our draw panel
		drawPanel = new JPanel() 
		{
            protected void paintComponent(Graphics g) 
			{
				super.paintComponent(g);
				paintLevel(g); //Call through to our external method to do it
			}
		};

		//Initilise level data
		newLevel(DEFAULT_LEVEL_WIDTH, DEFAULT_LEVEL_HEIGHT);
		//loadLevel(Cache.getClassLoader().getResourceAsStream("levels/empty.xml"));
		
		JScrollPane scroller = new JScrollPane(drawPanel);
 		add(scroller, BorderLayout.CENTER);
		
		drawPanel.setBackground(BACKGROUND);
	}

	public void loadLevel(File levelFile)
	{
		try
		{
			loadLevel(new FileInputStream(levelFile));
		}
		catch(FileNotFoundException e)
		{
			System.err.println(e);
		}
	}
	
	public void newLevel(int width, int height)
	{
		try
		{
			LevelLoader levelData = new XMLLevelLoader2(
				Cache.getClassLoader().getResourceAsStream("levels/empty.xml"), null);

			theLevel = new ArrayMap(width, 
								height,
								levelData.getLayers());
								
			drawPanel.setPreferredSize(new Dimension((int)(levelData.getWidth() * scale), (int)(levelData.getHeight())));
			
			theLevel.add(levelData.getKeen());
			keen = levelData.getKeen();
			
			visibleLayers = theLevel.getLayers();
			selecedSprites.clear();
			drawPanel.revalidate();
			repaint();
		}
		catch(IOException e)
		{
			e.printStackTrace();
			System.err.println("Cant load level");
			System.exit(0);
		}
	}
		
	
	private void loadLevel(InputStream file)
	{
		try
		{
			LevelLoader levelData = new XMLLevelLoader2(file, null);

			theLevel = new ArrayMap(levelData.getWidth(), 
								levelData.getHeight(),
								levelData.getLayers());
								
			drawPanel.setPreferredSize(new Dimension((int)(levelData.getWidth() * scale), (int)(levelData.getHeight())));

			Iterator it = levelData.getSprites().iterator();
			while (it.hasNext())
			{
				theLevel.add((Sprite)it.next());
			}
			
			theLevel.add(levelData.getKeen());
			keen = levelData.getKeen();
			visibleLayers = theLevel.getLayers();
			selecedSprites.clear();
			drawPanel.revalidate();
			repaint();
		}
		catch(IOException e)
		{
			e.printStackTrace();
			System.err.println("Cant load level");
			System.exit(0);
		}
	}
	
	public KeenActor getKeen()
	{
		return keen;
	}
	
	public void setScale(double newScale)
	{
		scale = newScale;
		drawPanel.setPreferredSize(new Dimension((int)(theLevel.getWidth() * scale),
												(int)(theLevel.getHeight() * scale)));
		drawPanel.revalidate();
	}
	
	public double getScale()
	{
		return scale;
	}
	
	public void addMouse(Tool toAdd)
	{
		drawPanel.addMouseListener(toAdd);
		drawPanel.addMouseMotionListener(toAdd);
	}
	
	public ArrayMap getLevel()
	{
		return theLevel;
	}
			
	public void setGridVisible(boolean visible)
	{
		gridVisible = visible;
		repaint();
	}
	
	public void setSnapToGrid(boolean snap)
	{
		if(snap != snapToGrid)
		{
			snapToGrid = snap;
			repaint();
		}
	}
	
	public boolean isSnapToGrid()
	{
		return snapToGrid;
	}
	
	public void setGridSize(int newGridSize)
	{
		gridSize = newGridSize;
		repaint();
	}
	
	public int getGridSize()
	{
		return gridSize;
	}
	
	public void setSpriteBoxVisible(boolean visible)
	{
		if(spriteBoxVisible != visible)
		{
			spriteBoxVisible = visible;
			repaint();
		}
	}
	
	public void setSelectedSpriteVisible(boolean visible)
	{
		selectedSpritesVisible = visible;
		repaint();
	}
	
	public void setSelectedSprites(Vector sprites)
	{
		selecedSprites = sprites;
		repaint();
	}
	
	/**
	 * Transforms an x, or y co-ordinate given from a mouse movement
	 * event from the level panel, into the co-ordinate system used
	 * by the levels ArrayMap.  You want to use this when adding in
	 * objects, or drawing boxes on the level
	 */
	public int transformToMap(int location)
	{
		int newLoc = (int)(location * (1/scale));
		
		if(snapToGrid)
		{
			newLoc = (int)Math.floor((double)newLoc / (double)gridSize)
					* gridSize;
		}
		
		return newLoc;
	}
	
	public int transformToPanel(int location)
	{
		return (int)(location * scale);
	}
	
	public int snapToGrid(int location)
	{
		if(snapToGrid)
		{
			location = (int)Math.floor((double)location / (gridSize * scale))
					* (int)(gridSize * scale);
		}
		return location;
	}	
	
	public void repaintLevel(int x, int y, int width, int height)
	{
		drawPanel.repaint(x, y, width, height);
	}
	
	public int transformNoSnap(int location)
	{
		return (int)(location * (1/scale));
	}
	
	public Rectangle getSpriteBox()
	{
		return spriteBox;
	}
	
	public void setVisibleLayers(String [] layers)
	{
		visibleLayers = layers;
		repaint();
	}
	
	public String [] getVisibleLayers()
	{
		return visibleLayers;
	}
	
	private void paintLevel(Graphics g)
	{
		drawArrayMap(g);
		
		//Draw grid
		if(gridVisible)
		{
			drawGrid(g);
		}
		if(spriteBoxVisible)
		{
			drawSpriteBox(g);
		}
		if(selectedSpritesVisible)
		{
			drawSelectionBoxes(g);
		}
	}
	
	private void drawArrayMap(Graphics g)
	{
		//get size
		Rectangle bounds = g.getClipBounds();
		
		g.setColor(Color.GRAY);
		g.fillRect(0,0,(int)(theLevel.getWidth() * scale), (int)(theLevel.getHeight() * scale));
		
		
		int x = (int)(bounds.x / scale);
		int y = (int)(bounds.y / scale);
		int width = (int)(bounds.width / scale);
		int height = (int)(bounds.height / scale);
		
		if(x >= (theLevel.getWidth()))
		{
			x = theLevel.getWidth() - 1;
		}
		if(y >= (theLevel.getHeight()))
		{
			y = theLevel.getHeight() - 1;
		}
	
		
		if((x + width) > theLevel.getWidth() )
		{
			width = theLevel.getWidth() - x ;
		}
		if((y + height) > theLevel.getHeight())
		{
			height = theLevel.getHeight() - y;
		}
		
		Iterator it = theLevel.iterator(x, y, width, height,visibleLayers);
        while(it.hasNext())
        {
        	((Sprite)it.next()).draw(g, 0, 0 , true, scale);
        }
	}
	
	private void drawGrid(Graphics g)
	{
		int width = (int)(theLevel.getWidth() * scale);
		int height = (int)(theLevel.getHeight() * scale);
		
		g.setColor(GRID_COLOR);
		int maxDistance = 0;
		if(width > height)
		{
			maxDistance = width;
		}
		else
		{
			maxDistance = height;
		}
		
		for(int distance = 0; distance < maxDistance; distance = distance + (int)(gridSize * scale))
		{
			//vertical
			g.drawLine(distance,0,distance,height);
			
			//horizontal
			g.drawLine(0,distance,width,distance);
		}
	}
	
	private void drawSpriteBox(Graphics g)
	{
		g.setColor(SPRITE_BOX_COLOR);
		g.drawRect((int)(spriteBox.x * scale),
					(int)(spriteBox.y * scale),
					(int)(spriteBox.width * scale),
					(int)(spriteBox.height * scale));
	}
	
	private void drawSelectionBoxes(Graphics g)
	{
		g.setColor(SELECTION_BOX_COLOR);
		
		Iterator it = selecedSprites.iterator();
		while(it.hasNext())
		{
			Sprite selectedSprite = (Sprite)it.next();
			g.drawRect((int)(selectedSprite.getX() * scale),
						(int)(selectedSprite.getY() * scale),
						(int)(selectedSprite.getWidth() * scale),
						(int)(selectedSprite.getHeight() * scale));
		}
	}		
}
