package keenGame.sprite;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.awt.Rectangle;

import keenGame.XmlElement;

/**
 * Actor extends class Sprite with the ability, to periodically change its self
 * over time.  Examples of Actors are lights which flash on or off, 
 * Monsters, and commander Keen.  You should only really extend this class if
 * you want to make somethign simple that doesnt move, such as a switch, or 
 * a flashing light, or computer screen.  If you wish to make something that
 * moves, you should definately extend MovingActor instead.
 * 
 * Actors have an UPDATE_PERIOD, an update() and setFrame() methods.
 * They are registered with the level in which they exist.  The level 
 * shal attempt to call the update() and setFrame() methods at intervals 
 * of no less than UPDATE_PERIOD milliseconds.
 * 
 * All code that modifies the Actor should be placed inside the update() method
 * The setFrame() method should analyse the current state of the Actor and 
 * update the image instance variable, to the correct image.
 * 
 * It is worth noting that the hitArea shape for Actors, and all their subclasses
 * can only be a Rectangle.
 */
public abstract class Actor extends Sprite 
{
    //Constants
    protected static long UPDATE_PERIOD = 0; //Should be zero for Actors, but other stuff might want to be more

    protected long nextUpdateTime;
    protected boolean enabled; //False if the object is dead and should be removed

	/**
     * Constructs an Actor.  When writing your own subclasses of Actor you
     * should always put in a call to this constructor with Super.  After 
     * this constructor has been called, you should define a hitArea, as well
     * as any other instance variables your Actor should need.
     * 
     * @param imageLocation - The location of an image (normally .png format) that
     * 		is used for this Actor
     * @param layer - The layer that this Actor should exist in
     * @param xValue - The x location of the Actor
     * @param yValue - The y location of the Actor
     * 
     * @see keenGame.sprite.Sprite#hitArea
     * @see keenGame.sprite.Sprite#Sprite(String, String, int, int)
     */
    public Actor(String layer, int xValue, int yValue)
    {
        super(layer, xValue,yValue);
        init();
    }
    
    public Actor(XmlElement spriteElement)
    {
    	super(spriteElement);
    	init();
    }
    
    private void init()
    {
        enabled = true;
        hitArea = new Rectangle(x, y, width, height);
        
        //Initlize our timers        
        nextUpdateTime = System.currentTimeMillis() + UPDATE_PERIOD;
    }
    
    /**
     * Returns true, if it is time for the update and setFrame methods
     * of the Actor to be called
     */ 
    public boolean timeForUpdate()
    {
    	return nextUpdateTime < System.currentTimeMillis();
    }
    
    /**
     * @return True if the Actor is "alive" false if it is "dead" and
     * should be removed from the level
     */
    public boolean isEnabled()
    {
    	return enabled;
    }
    
    /**
     * @return the hitArea of the Actor
     * @see keenGame.sprite.Sprite#hitArea
     */
    public Rectangle getArea()
    {
    	return (Rectangle)hitArea;
    }

	/**
	 * All code that updates the state of the Actor should be placed inside
	 * this method.  The level will attempt to call this method at intervals 
	 * of no less than UPDATE_PERIOD milliseconds.   Example of 
	 * things that should be placed inside the update method:
	 * <ul>
	 * 	<li>Code to make MovingActors move</li>
	 *  <li>Code to make automatic turrets spawn bullets</li>
	 * 	<li>Code that turns the state of a flashing light to On or Off</li>
	 * </ul>
	 * 
	 * @see #UPDATE_PERIOD
	 * @see #setFrame()
	 */
    public abstract void update();

	/**
	 * This method should analyze the current state of the Actor and update
	 * the image instance variable, to display the correct image.
	 * 
	 * The level will attempt to call this method at intervals 
	 * of no less than UPDATE_PERIOD milliseconds.  Example of 
	 * things that should be placed inside the update method:
	 * <ul>
	 * 	<li>Code to set the correct frame of animation for a monster</li>
	 * 	<li>Code to analyse the state of a flashing light, and set the
	 * 		image to be turned on, or turned off, depending on the state</li>
	 * 	<li>Code to display the correct animation frame for candy</li>
	 * </ul>
	 * 
	 * @see #UPDATE_PERIOD
	 * @see #update()
	 * @see keenGame.sprite.Sprite#image
	 */
    public abstract void setFrame();
}