package keenGame.sprite;

import keenGame.Level;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * A bullet is ths abstract base class for all bullets
 * in the Cranium engine.  It is provided so other Sprites
 * can provide a default hasHit implemenation for all bullets
 * should they want to.
 * 
 * Bulets also provide the state EXPLODING and a default implemntation
 * for hasHit which cause them to explode, upon hitting a sprite.
 */
public abstract class Bullet extends MovingActor
{
	public static final int EXPLODING = 5;
	
	protected int changeFrameCount = 0;
	protected int frame = 0;

	/**
     * Constructs an Bullet.  When writing your own subclasses of Bullet you
     * should always put in a call to this constructor with Super.  After 
     * this constructor has been called, you should define a hitArea, as well
     * as any other instance variables yourBullet should need.
     * 
     * @param imageLocation - The location of an image (normally .png format) that
     * 		is used for this Bullet
     * @param layer - The layer that this Bullet should exist in
     * @param hitLayers - An array of strings containing the layers which contain
     * 	Sprites with which this Bullet can collide
     * @param xValue - The x location of the Bullet
     * @param yValue - The y location of the Bullet
     * 
     * @see keenGame.sprite.Sprite#hitArea
     * @see keenGame.sprite.Actor#Actor(String, String, int, int)
     */
    public Bullet(String layer, String [] hitLayers, Level myWorld,
    					int xValue, int yValue)    
    {
        super(layer, hitLayers, myWorld, xValue,yValue);
    }
    
    /**
     * Default implemenation for hitting a sprite.  Changes the
     * Bullets state to EXPLODING
     * @return true
   	 * @see keenGame.sprite.Sprite#hasHit(Sprite)
     */
    public boolean hasHit(Sprite hitSprite)
    {
		if(State != EXPLODING)
		{
    		State = EXPLODING;
    		changeFrameCount = 0;
    		frame = 0;
    	}
    	return true;
    }


    /**
     * Default implemenation for hitting another Bullet.
     * Does nothing.
     * 
     * @return false
   	 * @see keenGame.sprite.Sprite#hasHit(Sprite)
     */
    public boolean hasHit(Bullet hitSprite)
    {
    	return false;
    }
}