package keenGame.sprite;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.awt.Image;
import java.awt.Rectangle;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import keenGame.Cache;
import keenGame.Level;
import keenGame.XmlElement;
import keenGame.sprite.keenActor.KeenActor;

/**
 * Candy is ths abstract base class for all Candy
 * in the Cranium engine.  It is provided so other Sprites
 * can provide a default hasHit implemenation for all Candy
 * should they want to.
 * 
 * All Candy cycles through two frames.  It switches frame
 * evey 1 second.  All candy will change to a number that
 * slowly drifts upwards when hit by Keen.  All you should realy
 * need to do, to implement your own candy, is implement a
 * collectedBy method, which gives the correct powerup to the
 * KeenActor, which collected the candy.  You would extend the
 * Candy class to implement anything that keen can pickup, such
 * as candy, keys, extra lives, and ammo
 */
public abstract class Candy extends MovingActor
{
	public static final int NONE = 0;
	public static final int NUMBER = 1;
	public static final int TRANSFORM = 2; //The candy was collected, and should change to a number
	
	private static final int UPDATE_PERIOD = 1000;
	private static final int NUMBER_TIME = 30;

	protected Image frame0;
	protected Image frame1;
	protected Image endFrame;
	
	private String image0, image1, endImage;
	
	private KeenActor collectActor;
	
	protected int State, dieCountdown;
	

	/**
     * Constructs an Candy.  When writing your own subclasses of Candy you
     * should always put in a call to this constructor with Super.  After 
     * this constructor has been called any instance variables your Candy
     * should need.
     * 
     * @param imageLocation - The location of an image (normally .png format) that
     * 		is used for this Candy
     * @param layer - The layer that this sprite should exist in
	 * @param myWorld - the Level in which the Candy exists
     * @param xValue - The x location of the Candy
     * @param yValue - The y location of the Candy
     * 
     * @see keenGame.sprite.Actor#Actor(String, String, int, int)
     */
	public Candy(String image0,String image1,String endImage,
				String layer, Level myWorld,
				 int xValue, int yValue)    
    {
        super(layer, new String[0], myWorld, xValue,yValue);
        init(image0, image1, endImage);
    }
    
    public Candy(XmlElement spriteElement, Level myWorld)
    {
    	super(spriteElement, myWorld);
    	
    	String newImage0 = spriteElement.getChild("image0").getText();
    	String newImage1 = spriteElement.getChild("image1").getText();
    	String endImage = spriteElement.getChild("endImage").getText();

    	init(newImage0, newImage1, endImage);
    }
    
    private void init(String image0, String image1, String endImage)
    {
        this.image0 = image0;
        this.image1 = image1;
        this.endImage = endImage;
        
        frame0 = Cache.getImage(image0);
        frame1 = Cache.getImage(image1);
        endFrame = Cache.getImage(endImage);
        image = frame0;
        State = NONE;
        
        width = frame0.getWidth(null);
	    height = frame0.getHeight(null);
        hitArea = new Rectangle(x, y, width, height);

    }
    
	protected Element addToElement(Document xmlDoc, Element element)
    {
    	super.addToElement(xmlDoc, element);
    	
    	//Graphic
    	Element graphicElement = xmlDoc.createElement("image0");
		graphicElement.appendChild(xmlDoc.createTextNode(image0));
		element.appendChild(graphicElement);

    	graphicElement = xmlDoc.createElement("image1");
		graphicElement.appendChild(xmlDoc.createTextNode(image1));
		element.appendChild(graphicElement);

    	graphicElement = xmlDoc.createElement("endImage");
		graphicElement.appendChild(xmlDoc.createTextNode(endImage));
		element.appendChild(graphicElement);

		
    	return element;
    }
     
    /**
     * If the candy has been touched by keen, moves upwards slowly, and
     * then disapares.  Otherwise does nothing
     */
    public void update()
    {
	    //Move
	    if(State == NUMBER)
	    {
	    	if(y == 0)
	    	{
	    		enabled = false;
	    	}
	    	else
	    	{
		    	attemptMove(x, y - 1);
	    	}

		    dieCountdown--;
		    if(dieCountdown == 0)
		    {
		    	enabled = false;
		    }
	    }
	    else if(State == TRANSFORM)
	    {
			//move to foreground
			String [] layers = myWorld.getLayers();
			String newLayer = layers[layers.length - 1];
			myWorld.changeLayer(this, newLayer);

	  		collectedBy(collectActor);
    		State = NUMBER;
    		dieCountdown = NUMBER_TIME;
    		image = endFrame;
    		nextUpdateTime = System.currentTimeMillis();
    	}
    	else
    	{
		    //update the timers
		    nextUpdateTime = System.currentTimeMillis() + UPDATE_PERIOD;
    	}
    }
    
    /**
     * Changes the image, and starts to move upwards.
     * 
     * @return false
   	 * @see keenGame.sprite.Sprite#hasHit(Sprite)
     */
    public boolean hasHit(KeenActor hitSprite)
    {
    	if(State == NONE)
    	{
    		State = TRANSFORM;
    		collectActor = hitSprite;
	  		nextUpdateTime = System.currentTimeMillis();
    	}

		return true;
    }

	/**
	 * Implement this method, to give keen the correct power up, 
	 * as well as any other misc stuff that needs doing when
	 * the cady is collected
	 */    
    protected abstract void collectedBy(KeenActor keen);

	/**
	 * @see keenGame.sprite.Actor#setFrame()
	 */    
    public void setFrame()
    {
    	//Set to the correct animation sequence
    	if(State == NONE)
    	{
    		if(image == frame0)
    		{
    			image = frame1;
    		}
    		else
    		{
    			image = frame0;
    		}
    	}
    	if(State == NUMBER)
    	{
    		image = endFrame;
    	}
    }
}
