package keenGame.sprite;

import java.awt.Point;

import keenGame.Level;
import keenGame.XmlElement;
import keenGame.sprite.monster.SpinningStars;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * Provides an abstract base classes for all Monsters, should
 * any class wish to provide a default implementation for
 * hasHit(Monster).
 * 
 * Monsters have an extra state STUNED, which they enter when
 * they are hit by a Bullet
 */
public abstract class Monster extends MovingActor
{
	public int STUNED = 5;
	
	protected int changeFrameCount = 0;
	
	/**
     * Constructs an Monster.  When writing your own subclasses of Monster you
     * should always put in a call to this constructor with Super.  After 
     * this constructor has been called, you should define a hitArea, as well
     * as any other instance variables yourBullet should need.
     * 
     * @param imageLocation - The location of an image (normally .png format) that
     * 		is used for this Monster
     * @param layer - The layer that this Monster should exist in
     * @param hitLayers - An array of strings containing the layers which contain
     * 	Sprites with which this Monster can collide
     * @param xValue - The x location of the Monster
     * @param yValue - The y location of the Monster
	 * @param myWorld - the Level in which the MovingActor exists
     * 
     * @see keenGame.sprite.Sprite#hitArea
     * @see keenGame.sprite.Actor#Actor(String, String, int, int)
     */
    public Monster(String layer, 
    					String [] hitLayers, Level myWorld,
    					int xValue, int yValue)    
    {
        super(layer, hitLayers, myWorld, xValue,yValue);
    }

    public Monster(XmlElement spriteElement, Level myLevel)
    {
    	super(spriteElement, myLevel);
    }

	/**
	 * Default implementation for hiting a sprite.
	 * 
	 * @return true
	 * @see keenGame.sprite.Sprite#hasHit(Sprite)
	 */
    public boolean hasHit(Sprite hitSprite)
    {
    	return true;
    }
    
    /**
     * Moves the Monster into the stunned state
     * 
	 * @return true
	 * @see keenGame.sprite.Sprite#hasHit(Sprite)
	 */
    public boolean hasHit(Bullet hitSprite)
    {
    	if(State != STUNED)
    	{
	    	State = STUNED;
	    	changeFrameCount = 0;
	    	myWorld.addSprite(new SpinningStars(getStarsLocation().x, getStarsLocation().y));
	    	return true;
    	}
    	else
    	{
	    	return false;
    	}
    }
    
    public boolean hasHit(Monster hitSprite)
    {
    	return false;
    }
    
    public boolean hasHit(Player hitSprite)
    {
    	return false;
    }
    
    protected abstract Point getStarsLocation();
}