package keenGame.sprite;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.awt.Rectangle;
import java.lang.reflect.Method;
import java.util.Iterator;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import keenGame.Cache;
import keenGame.Level;
import keenGame.XmlElement;

import keenGame.sprite.tile.Pole;
import keenGame.sprite.tile.PoleBottom;
import keenGame.sprite.tile.PoleTop;
import keenGame.sprite.tile.Switch;
import keenGame.sprite.tile.ThinFloor;


/**
 * MovingActor is an Actor, that is able to move its self about the game
 * world.  If you want to make an Sprite that can move, extend MovingActor
 * and make use of the attemptMove, directionMove, walk, and fall methods.
 * If you feel you need to make an Sprite that can move, yet should not extend
 * MovingActor, then you should probably think again about your design
 * 
 * As well as containg methods that allow it to move MovingActor is also
 * capible of notifying MovementListeners about its movement.  It also defines
 * several default hasHit methods, defining basic responsies for collisions
 * with various types of Sprite.
 * 
 * @see #attemptMove(int, int)
 * @see #directionMove(int, int)
 * @see #fall(double)
 * @see #walk(int, int)
 */
public abstract class MovingActor extends Actor
{
    private static final int MAX_NUMBER_OF_HITS = 50; //The maximum number of collisions that can occur eax move
    private static final int MAX_FALL_SPEED = 9; //The fastes an object can possiably fall
    private static final double ACCLERATON = 1.2; //Every time we fall, our speed is multipled by this
    
    //public static final int DEFAULT_LAYER =2;
    
    //Directions
    public static final int NONE = 0;
    public static final int NORTH = 1;
    public static final int NORTH_EAST = 2;
    public static final int EAST = 3;
    public static final int SOUTH_EAST = 4;
    public static final int SOUTH = 5;
    public static final int SOUTH_WEST = 6;
    public static final int WEST = 7;
    public static final int NORTH_WEST = 8;
    
    //States
    public static final int STATIONARY = 0;
    public static final int WALKING_LEFT = 1;
    public static final int WALKING_RIGHT = 2;
    public static final int FALLING = 3;
    public static final int FLYING = 4;
    
	//Must be one of the above defined states, or a similar state defined in a subclass
	protected int State = STATIONARY;
	
	/**
	 * The layers with objects with which this MovingActor can collide
	 */
	protected String [] hitLayers;
	
	/**
	 * The level in which this actor exists
	 */
	protected Level myWorld;

	//Stuff to do with collisions
    //protected Iterator collisionIterator;
    
    //Cache
    //Used for black magic with colisions
  	private Sprite [] callArray = new Sprite[MAX_NUMBER_OF_HITS];
    private Object [] hitObject = new Object[1];
	private LinePointIterator line = new LinePointIterator();
    private Sprite hitSprite;
    private Method handleMethod;
    private Rectangle simRect = new Rectangle();
   	private Sprite collisionSprite;
   	
   	//Who wants to know about our movement
   	private static final int MAX_LISTENERS = 5;
   	private MovementListener [] listeners = new MovementListener[MAX_LISTENERS];
   	private int oldX, oldY;
   	private int numListeners = 0;
    
	/**
     * Constructs an MovingActor.  When writing your own subclasses of MovingActor you
     * should always put in a call to this constructor with Super.  After 
     * this constructor has been called, you should define a hitArea, as well
     * as any other instance variables your MovingActor should need.
     * 
     * @param imageLocation - The location of an image (normally .png format) that
     * 		is used for this MovingActor
     * @param layer - The layer that this MovingActor should exist in
     * @param hitLayers - An array of strings containing the layers which contain
     * 	Sprites with which this MovingActor can collide'
     * @param myWorld - the Level in which the MovingActor exists
     * @param xValue - The x location of the MovingActor
     * @param yValue - The y location of the MovingActor
     * 
     * @see keenGame.sprite.Sprite#hitArea
     * @see keenGame.sprite.Actor#Actor(String, String, int, int)
     */
    public MovingActor(String layer, 
    					String [] hitLayers, Level myWorld,
    					int xValue, int yValue)    
    {
        super(layer, xValue,yValue);
        init(hitLayers, myWorld);
    }
    
    public MovingActor(XmlElement spriteElement, Level myWorld)
    {
    	super(spriteElement);
    	
    	//get layers
		XmlElement [] hitLayerElements = spriteElement.getChild("hitLayers").getChildren();
    	String [] newHitLayers = new String[hitLayerElements.length];
    	
    	for(int i = 0; i < newHitLayers.length; i++)
    	{
    		newHitLayers[i] = hitLayerElements[i].getText();
    	}
    	
    	init(newHitLayers, myWorld);
    }
    
    private void init(String [] hitLayers, Level myWorld)
    {  
        this.hitLayers = hitLayers;
        this.myWorld = myWorld;
        //simRect.width = width;
		//simRect.height = height;
		oldX = x;
		oldY = y;
    }
    
    protected Element addToElement(Document xmlDoc, Element element)
    {
    	super.addToElement(xmlDoc, element);
    	
		//add parent node
		Element hit = xmlDoc.createElement("hitLayers");
		element.appendChild(hit);
		
		//loop around, addin in
		for (int i = 0; i < hitLayers.length; i++)
		{
			Element layerElement = xmlDoc.createElement("layer");
			layerElement.appendChild(xmlDoc.createTextNode(hitLayers[i]));
			hit.appendChild(layerElement);
		}
		return element;
    }
	
	/**
     * Moves as close as possible to the specified points, before it hits an object
     * that will stop this from moving.
     * This method should be as optimsed as possible, since it gets called many, many
     * times per update, and currently slows down the game no end ...
     *
     * @return boolean true if it made it to the specified position
     *			false if it was stoped before it could get there
     * @see keenGame.sprite.Sprite#hasHit(Sprite)
     */

    protected boolean attemptMove(int newX, int newY)
    {
		boolean hasCalled = false;
   		int calledSize = 0;
    	
        if((newX == x) && (newY == y))
   		{
   			//then your already there ... cant really do much for you
   			return true;
   		}
   		
   		int minX, maxX, minY, maxY;

		//Compensate for moving up and/or to the left
    	if(x < newX)
    	{
    		minX = x;
    		maxX = newX + width;
    	}
    	else
    	{
    		minX = newX;
    		maxX = x + width;
    	}
    	if(y < newY)
    	{
    		minY = y;
    		maxY = newY + height;
    	}
    	else
    	{
    		minY = newY;
    		maxY = y + height;
    	}
    	
    	//Set up our collisionIterator
    	//((CellMap.SpriteIterator)collisionIterator).setLocation(minX,minY,maxX-minX, maxY-minY);
		boolean collision = false;
		
		Iterator collisionIterator;
		
    	if(hitLayers.length > 0)
    	{
	    	collisionIterator = myWorld.iterator(minX,minY,maxX-minX, maxY-minY, hitLayers);
	    	
	
	    	//Set up the simulation rectangle
			simRect.x = newX;
			simRect.y = newY;
			simRect.width = width;
			simRect.height = height;
	    	
			//loop through the iterator checking for a collision that will stop us at the new co-ords
			while(collisionIterator.hasNext() && !collision)
			{
				collisionSprite = (Sprite)(collisionIterator.next());
				if(collisionSprite != this)
				{
					collision = collisionSprite.isCollision(simRect);
				}
			}
    	}

		if(!collision)  //There is no collision, we are home free!
		{
			((Rectangle)hitArea).x = ((Rectangle)hitArea).x + (newX - x);
			((Rectangle)hitArea).y = ((Rectangle)hitArea).y + (newY - y);
			x = newX;
			y = newY;
			notifyMoved();
			
			return true;
		}
		else //Bugger ... more work for me :-(
		{
   			int tempX = x;
   			int tempY = y;
   			line.initIterator(x,y,newX,newY);
   			boolean stoped = false;

   			while((x != newX) || (y != newY))
   			{
   				//((CellMap.SpriteIterator)collisionIterator).setLocation(tempX,tempY,width, height);
				collisionIterator = myWorld.iterator(tempX,tempY,width, height, hitLayers);
		
				simRect.x = tempX;
				simRect.y = tempY;
    	
				//loop through the iterator checking for a collision at the new co-ords
				while(collisionIterator.hasNext()) 
				{
					collisionSprite = (Sprite)(collisionIterator.next());
					if((collisionSprite != this) && (collisionSprite.isCollision(simRect)))
					{	//then we have hit.  Deal with it
						hasCalled = false; //Check if we have already called it
	   					for(int i = 0; ((i<calledSize) && (!hasCalled)); i++)
	   					{
	   						if(callArray[i] == collisionSprite) //then already called
	   						{
	   							hasCalled = true;
	   						}
	   					}
	
	   					if(!hasCalled) //then we better call it
	   					{
	   						try //Some black magic to call the correct hasHit() method
	   						{
								//hitObject.hasHit(moveActor);
		   						handleMethod = Cache.getHandleMethod(collisionSprite.getClass(), this.getClass());
			   					hitObject[0] = this;
		   						handleMethod.invoke(collisionSprite, hitObject);

		   						//stop = moveActor.hasHit(hitObject);
			   					handleMethod = Cache.getHandleMethod(this.getClass(),collisionSprite.getClass());
		   						hitObject[0] = collisionSprite;
		   						
		   						if(((Boolean)(handleMethod.invoke(this, hitObject))).booleanValue())
		   						{ //Check to see if we should stop
		   							stoped = true;
		   						}
		 					}
	   						catch(Exception e)
	   						{
	   							e.printStackTrace();
	   							System.exit(0);
	   						}
							callArray[calledSize] = collisionSprite;
							calledSize++;
						}
					}
   				}
				if(stoped)
				{
					return false; //We didnt make it
				}

				//If we made it through all that, we obvisly should move
				((Rectangle)hitArea).x = ((Rectangle)hitArea).x + (tempX - x);
				((Rectangle)hitArea).y = ((Rectangle)hitArea).y + (tempY - y);
				x = tempX;
				y = tempY;
				notifyMoved(); //And we better tell eveybody about it
   				
   				//calculate next position
   				line.next();
   				tempX = line.getX();
   				tempY = line.getY();
			}
			//Woop, we actually got there
			return true;
		}
    }

    /**
     * Attempts to move in the specified distance, in the specified
     * direction
     * 
     * @param moveDistance - The number of pixels to move
     * @param direction - The Direction to move.  One of the compas
     * 	direction constants in MovingActor
     * @return boolean true if it made it to the specified position
     *			false if it was stoped before it could get there
     * @see #attemptMove(int, int)
     */
    protected boolean directionMove(int moveDistance, int direction)
    {
    	switch(direction) //This case statement can be optmised more
        {
            case NORTH:
                return attemptMove(x,y - moveDistance);
            case NORTH_EAST:
                return attemptMove(x + moveDistance,y - moveDistance);
            case EAST:
                return attemptMove(x + moveDistance,y);
            case SOUTH_EAST:
                return attemptMove(x + moveDistance,y + moveDistance);
            case SOUTH:
                return attemptMove(x,y + moveDistance);
            case SOUTH_WEST:
                return attemptMove(x - moveDistance,y + moveDistance);
            case WEST:
                return attemptMove(x - moveDistance,y);
            case NORTH_WEST:
                return attemptMove(x -  moveDistance,y - moveDistance);
        }
        return true;
    }
    
    /**
     * Walks in the specified direction
     * @returns true if we are standing on something
     *			false if we are standing on nothing ... looks down ... blinks eyes ... ARGHHhhhhhhhhhh.....
     * @see #attemptMove(int, int)
     */
    protected boolean walk(int direction, int moveDistance)
    {
    	if(direction != STATIONARY)
    	{
	    	//Up and across
	    	if(direction == WALKING_LEFT)
	    	{
	    		attemptMove(x - moveDistance, y - 2*moveDistance);
	    	}
	    	else //We must be walking right
	    	{	
	    		attemptMove(x + moveDistance, y - 2*moveDistance);
	    	}
	    	
	    	//Down
	    	return !attemptMove(x, y + (4*moveDistance));
    	}
    	return false;
    }
    
    /**
     * plumits to earth.
     * @return the new speed of falling. 0 if we hit something and stoped
     * @see #attemptMove(int, int)
     */
    protected double fall(double fallSpeed)
    {
    	if(attemptMove(x,y + (int)fallSpeed))
    	{
    		if(fallSpeed < MAX_FALL_SPEED)
    		{
    			fallSpeed = fallSpeed * ACCLERATON;
    		}
   		}
   		else
   		{
   			fallSpeed = 0;
   		}
   		return fallSpeed;
   	}
   	
   	/**
   	 * Notifies all movementListeners, that this actor has moved
   	 * This should be called, EVERY time the actor modifies its X 
   	 * or Y position, or trouble will ensue...
   	 */
   	protected void notifyMoved()
	{
		//Create our event
		MovementEvent move = new MovementEvent(this, oldX, oldY, x, y);
		oldX = x;
		oldY = y;
		
		//Iterate through, and throw the event
		for(int i = 0; i < numListeners; i++)
		{
			listeners[i].movementPerformed(move);
		}
	}	
   
   /**
    * Adds an object that listens to the Actors movement.  The
    * MovingActor should be, as a minimum added into any ArrayMaps
    * it is contained in
    */
   	public void addMovementListener(MovementListener newListener)
   	{
   		if(numListeners == MAX_LISTENERS)
   		{
   			throw new UnsupportedOperationException("Cant add more than " + MAX_LISTENERS + " listeners");
   		}
   		listeners[numListeners] = newListener;
   		numListeners++;
   	}

   /**
    * Removes an object that listens to the Actors movement.
    */
   	public void removeMovementListener(MovementListener removeListener)
   	{
   		MovementListener[] newListeners = new MovementListener [MAX_LISTENERS];
   		numListeners = 0;
   		for(int i = 0; i < MAX_LISTENERS; i++)
   		{
   			if((listeners[i] != removeListener) && (listeners[i] != null))
   			{
   				newListeners[numListeners] = listeners[i];
   				numListeners++;
   			}
   		}
   		listeners = newListeners;   		
   	}
   	
   	protected int stringToDirection(String directionString)
	{
		int direction = MovingActor.NONE;
		if(directionString.equals("north"))
		{
			direction = MovingActor.NORTH;
		}
		else if(directionString.equals("north east"))
		{
			direction = MovingActor.NORTH_EAST;
		}
		else if(directionString.equals("east"))
		{
			direction = MovingActor.EAST;
		}
		else if(directionString.equals("south east"))
		{
			direction = MovingActor.SOUTH_EAST;
		}
		else if(directionString.equals("south"))
		{
			direction = MovingActor.SOUTH;
		}
		else if(directionString.equals("south west"))
		{
			direction = MovingActor.SOUTH_WEST;
		}
		else if(directionString.equals("west"))
		{
			direction = MovingActor.WEST;
		}
		else if(directionString.equals("north west"))
		{
			direction = MovingActor.NORTH_WEST;
		}
		return direction;
	}	
   	
   	protected String directionToString(int Direction)
    {
    	String direction = null;
    	
		switch(Direction) //This case statement can be optmised more
        {
            case MovingActor.NORTH:
            	direction = "north";
            	break;
            case MovingActor.NORTH_EAST:
            	direction = "north east";
            	break;
            case MovingActor.EAST:
            	direction = "east";
            	break;
            case MovingActor.SOUTH_EAST:
            	direction = "south east";
            	break;
            case MovingActor.SOUTH:
            	direction = "south";
            	break;
            case MovingActor.SOUTH_WEST:
            	direction = "south west";
            	break;
            case MovingActor.WEST:
            	direction = "west";
            	break;
            case MovingActor.NORTH_WEST:
            	direction = "north west";
            	break;
        }
        return direction;
    }
   	
   	/**
   	 * Default behavour for hitting a Pole.
   	 * Keeps moving
   	 * 
   	 * @return false
   	 * @see keenGame.sprite.Sprite#hasHit(Sprite)
   	 */
   	public boolean hasHit(Pole hitSprite)
    {
    	return false;
    }

   	/**
   	 * Default behavour for hitting a Pole.
   	 * Keeps moving
   	 * 
   	 * @return false
   	 * @see keenGame.sprite.Sprite#hasHit(Sprite)
   	 */
   	public boolean hasHit(PoleTop hitSprite)
    {
    	return false;
    }

   	/**
   	 * Default behavour for hitting a Pole.
   	 * Keeps moving
   	 * 
   	 * @return false
   	 * @see keenGame.sprite.Sprite#hasHit(Sprite)
   	 */
    public boolean hasHit(PoleBottom hitSprite)
    {
    	return false;
    }    

   	/**
   	 * Default behavour for hitting a Switch.
   	 * Keeps moving
   	 * 
   	 * @return false
   	 * @see keenGame.sprite.Sprite#hasHit(Sprite)
   	 */
    public boolean hasHit(Switch hitSprite)
    {
    	return false;
    } 
   
    public boolean hasHit(Hint hitSprite)
    {
    	return false;
    }
    
    public boolean hasHit(Platform hitSprite)
    {
    	return false;
    }

    public boolean hasHit(ThinFloor hitSprite)
    {
    	return false;
    }

    public boolean hasHit(Candy hitSprite)
    {
    	return false;
    }
}