package keenGame.sprite;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import keenGame.Cache;
import keenGame.Level;
import keenGame.XmlElement;
import keenGame.sprite.keenActor.KeenActor;

/**
 * Provides an abstract base classes for all Platforms, should
 * any class wish to provide a default implementation for
 * hasHit(Platform).
 * 
 * A platform can be stood on by a KeenActor.  When a KeenActor
 * is standing on a platform, he will move with the platform
 */
public abstract class Platform extends MovingActor
{
	private String graphic;
	/**
     * Constructs an Platform.  When writing your own subclasses of Platform you
     * should always put in a call to this constructor with Super.  After 
     * this constructor has been called, you should define a hitArea, as well
     * as any other instance variables yourBullet should need.
     * 
     * @param imageLocation - The location of an image (normally .png format) that
     * 		is used for this Platform
     * @param layer - The layer that this Platform should exist in
     * @param hitLayers - An array of strings containing the layers which contain
     * 	Sprites with which this Platform can collide
     * @param xValue - The x location of the Platform
     * @param yValue - The y location of the Platform
     * 
     * @see keenGame.sprite.Sprite#hitArea
     * @see keenGame.sprite.Actor#Actor(String, String, int, int)
     */
    public Platform(String imageLocation, String layer, 
    					String [] hitLayers, Level myWorld,
    					int xValue, int yValue)    
    {
        super(layer, hitLayers, myWorld, xValue,yValue);
        init(imageLocation);
    }
    
    public Platform(XmlElement spriteElement, Level myWorld)
    {
    	super(spriteElement, myWorld);
    	
    	//get image
   		String newGraphic = spriteElement.getChild("graphic").getText();
    	
    	init(newGraphic);
    }
    
    private void init(String imageLocation)
    {
    	graphic = imageLocation;

    	image = Cache.getImage(imageLocation);
    	width = image.getWidth(null);
    	height = image.getHeight(null);
    }
    
    protected Element addToElement(Document xmlDoc, Element element)
    {
    	super.addToElement(xmlDoc, element);
    	
		//graphic
    	Element graphicElement = xmlDoc.createElement("graphic");
		graphicElement.appendChild(xmlDoc.createTextNode(graphic));
		element.appendChild(graphicElement);
		
    	return element;
    }

	/**
	 * TODO:  Replace this with movement listeners when KeenActor is
	 * refactored
	 */
    public abstract boolean connect(KeenActor standKeen);

	/**
	 * TODO:  Replace this with movement listeners when KeenActor is
	 * refactored
	 */
    public abstract void disconnect();

	/**
	 * Does nothing as platforms usually dont have different frames
	 * 
	 * @see keenGame.sprite.Actor#setFrame()
	 */    
   	public void setFrame()
    {
    }
}
