package keenGame.sprite;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import keenGame.Cache;
import keenGame.XmlElement;

/**
 * Eveything inside the game world is a sprite.  It defines the
 * basic functionality needed by all objects that exist in the world
 * It has an x and y location, a layer, and an image.  When asked to
 * draw its self, the sprite will draw its image at the correct location
 * on the provided graphics object.   
 * 
 * Sprites also have facilities for calculating if a collision has
 * occured, and dealing with a collision once it has happend.  When implemeting
 * your own sub-classes of Sprite it is important to understand how the Cranium
 * handles collision.  Read the comments for Sprite.hasHit(Sprite) to get a better over
 * view.
 * 
 * If you wish to add your own objects into the game, then it would
 * be wise to consider extending one of the more fully featured subclasses
 * of Sprite, such as Actor, or MovingActor.  You should only extend sprite
 * if you wish to make a very simple static object, that does not move,
 * or change in any way over time.
 * @see #hasHit(Sprite)
 * @see keenGame.sprite.Actor
 * @see keenGame.sprite.MovingActor
 */
public abstract class Sprite
{
	/**
	 * A shape, that will be used in collision detection.  The shape should
	 * encompas all areas of the Sprite that can be collided with.  Note that
	 * the Shapes origin should be close to the sprites X and Y location, and
	 * should not originate at 0,0
	 */
	protected Shape hitArea;
	
    protected int x, y;
    protected int width = 0, height = 0;
    
    /**
     * The layer that the Sprite lives in
     * @see keenGame.ArrayMap
     */
    protected String layer;

    protected Image image;
    
    private Point currentLoc = new Point();
    
    /**
     * Constructs a sprite.  When writing your own subclasses of Sprite you
     * should always put in a call to this constructor with Super.  After 
     * this constructor has been called, you should define a hitArea, as well
     * as any other instance variables your Sprite should need.
     * 
     * @param imageLocation - The location of an image (normally .png format) that
     * 		is used for this sprite
     * @param layer - The layer that this sprite should exist in
     * @param xValue - The x location of the Sprite
     * @param yValue - The y location of the Sprite
     * 
     * @see #hitArea
     */
    public Sprite(String layer, int xValue, int yValue)
    {
    	init(layer, xValue, yValue);
    }
    
    public Sprite(XmlElement spriteElement)
    {
    	String newLayer = spriteElement.getChild("spriteLayer").getText();
    	int newX = spriteElement.getChild("x").getValue();
    	int newY = spriteElement.getChild("y").getValue();
    	init(newLayer, newX, newY);
    }
    
    private void init(String layer, int xValue, int yValue)
    {
        //Setup other stuff
        x = xValue;
        y = yValue;
        this.layer = layer;
    }
    
    public abstract Element toXml(Document xmlDoc);
    
    protected Element addToElement(Document xmlDoc, Element element)
    {
    	//x
    	Element xElement = xmlDoc.createElement("x");
    	xElement.appendChild(xmlDoc.createTextNode(Integer.toString(x)));
    	element.appendChild(xElement);
    	
    	//y
    	Element yElement = xmlDoc.createElement("y");
    	yElement.appendChild(xmlDoc.createTextNode(Integer.toString(y)));
    	element.appendChild(yElement);
    	 	
    	//layer
    	Element spriteLayer = xmlDoc.createElement("spriteLayer");
    	spriteLayer.appendChild(xmlDoc.createTextNode(layer));
    	element.appendChild(spriteLayer);
    	
    	return element;
    }
    
    public int getX()
    {
    	return x;
    }
    
    public int getY()
    {
    	return y;
    }
    
    public int getWidth()
    {
    	return image.getWidth(null);
    }
    
    public int getHeight()
    {
    	return image.getHeight(null);
    }
    
    public Point getLocation()
    {
    	currentLoc.x = x;
    	currentLoc.y = y;
    	return currentLoc;
    	//return new Point(x,y);
    }
    
	/**
	 * Returns the layer.
	 * @return String
	 */
	public String getLayer()
	{
		return layer;
	}
	
	/**
	 * Changes the name of the layer the sprite is currently in
	 * 
	 * @param newLayer - The new name of the layer the sprite is currently
	 * 					in
	 */
	public void setLayer(String newLayer)
	{
		layer = newLayer;
	}

	/**
	 * The Cranium engine deals with the effects of collision 
	 * via the hasHit methods contained in each subclass of sprite.
	 * Define hasHit methods for which ever kind of Sprite you wish
	 * to implement special events on collision.  These will be called
	 * pollymorphicly whenever a collision with that particular kind of
	 * sprite occurs.  For example, you may be writing a special kind
	 * of blast shot through a gun.  For normal monsters you want
	 * the blast to explode when it hits them.  For MirrorMonsters
	 * you want it to bounce off them.  You would define two hasHit
	 * methods:
	 * <pre>
	 * public boolean hasHit(Monster hitSprite)
	 * ...
	 * public boolean hasHit(MirrorMonster hitSprite)
	 * ...
	 * </pre>
	 * In the first method, you would write code which makes the blast
	 * explode.  In the second, you would write the code that makes the
	 * blast reflect back.  The correct method will be called automagicly
	 * even if you subclass your blast class.
	 * 
	 * The boolean returned by hasHit methods, only applies if this Sprite
	 * is the one that was moving when the collision occured.  If true is
	 * returned, it indicates that this Sprite should stop moving once the
	 * collision has occured.  If false is returned, it indicates that the 
	 * sprite should keep on moveing.
	 * 
	 * None of the hasHit methods should change the location, or the image of the
	 * object.  Instead they should set some sort of instance variable to true, which
	 * is read during the update method
	 * 
	 * The magic behind the hasHit methods is acheived using reflection.  Look
	 * at the Cache.getHandleMethod(...) and MovingActor.attemptMove(...) methods
	 * for more information.
	 * 
     * This default hasHit method defines what happens when there is a collision 
     * with any object which has Sprite as its super class (This should be everything!)
     * It does nothing, and allows this sprite to continue moving by returning true
     *
     * @param hitSprite - The sprite with which the collision has occured
     * @returns boolean - True if you wish the world to stop this object
     *      			  as soon as the collision has occured
     * 
     * @see keenGame.Cache#getHandleMethod(Class, Class)
     * @see keenGame.sprite.MovingActor#attemptMove(int, int)
     */
    public boolean hasHit(Sprite hitSprite)
    {
    	return true;
    }

	/**
	 * Draws the Sprite to the provided graphics.  This will
	 * take into account the provided offsets, to draw the
	 * Sprite at the correct position.
	 * 
	 * @param  xOffset - The distance between the upper left
	 * 	corner of the graphics object, and the left-most border
	 * 	of the level in which the sprite exists.
	 * @param  yOffset - The distance between the upper left
	 * 	corner of the graphics object, and the top border
	 * 	of the level in which the sprite exists.
	 */
    public void draw( Graphics drawTo, int xOffset, int yOffset)
    {
    	draw(drawTo, xOffset, yOffset, false, 1.0);
    }
	
	
    public void draw( Graphics drawTo, int xOffset, int yOffset, boolean debug, double scale)
    {
		int screenX = x - xOffset;
        int screenY = y - yOffset;
        
        //if(!debug)
        //{
        	//drawTo.setClip(screenX,screenY,image.getWidth(null),image.getHeight(null));
        //}
        
		if(scale == 1.0)
		{
	        drawTo.drawImage(image,screenX,screenY, null); //Is this the best way to put it to the screen???
		}
		else
		{
	        drawTo.drawImage(Cache.getScaledImage(image,scale) ,(int)(screenX * scale) ,(int)(screenY * scale), null); //Is this the best way to put it to the screen???
	    }
    }

	/**
	 * calculates if there is a colision between this Sprite 
	 * and the provided Rectangle
	 * 
	 * @param moveingSprite - A check is done to see if this
	 * 	Rectangle intersects with this Sprites hitArea
	 * @return true if this Sprites hitArea intersects with
	 * 	movingSprite
	 * 
	 * @see #hitArea
	 */
    public boolean isCollision(Rectangle movingSprite)
    {
		if(hitArea == null)
		{
			return false;
		}
		else
		{
			return hitArea.intersects(movingSprite);
		}
    }
    
    /**
     * Used by the editor for moving sprites around.  You should not
     * be calling this method.  Sprites should move themselves.  Try
     * looking at MovingActor.  Unless you really know what you are
     * doing calling this method may cause crashes
     */
    public void setX(int x)
    {
    	this.x = x;
    }

    /**
     * Used by the editor for moving sprites around.  You should not
     * be calling this method.  Sprites should move themselves.  Try
     * looking at MovingActor.  Unless you really know what you are
     * doing calling this method may cause crashes
     */
    public void setY(int y)
    {
    	this.y = y;
    }    
}
