package keenGame.sprite.hint;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.awt.Rectangle;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import keenGame.Cache;
import keenGame.XmlElement;
import keenGame.sprite.Hint;
import keenGame.sprite.MovingActor;

/**
 * A DirectionHint sugests that a MovingActor that hits it, move in
 * a compas direction, represented by the DirectionHint.  DirectionHints
 * are used to control the paths that platforms take
 */
public class DirectionHint extends Hint
{
	private int Direction;

	/**
     * Constructs an DirectionHint.  
     * 
     * @param layer - The layer that this DirectionHint should exist in
     * @param xValue - The x location of the DirectionHint
     * @param yValue - The y location of the DirectionHint
     * @param Direction - The direction a MovingActor should head in upon colliding
     * 	with this DirectionHint
     * 
     * @see keenGame.sprite.Actor#Actor(String, String, int, int)
     */
    public DirectionHint(String layer, int xValue, int yValue, int Direction)    
    {
    	super(layer, xValue,yValue);
    	init(Direction);
    }
    
    public DirectionHint(XmlElement spriteElement)
    {
    	super(spriteElement);
    	init(parseDirection(spriteElement.getAttribute("direction")));
    }
    
    private int parseDirection(String directionString)
	{
		int direction = MovingActor.NONE;
		if(directionString.equals("north"))
		{
			direction = MovingActor.NORTH;
		}
		else if(directionString.equals("north east"))
		{
			direction = MovingActor.NORTH_EAST;
		}
		else if(directionString.equals("east"))
		{
			direction = MovingActor.EAST;
		}
		else if(directionString.equals("south east"))
		{
			direction = MovingActor.SOUTH_EAST;
		}
		else if(directionString.equals("south"))
		{
			direction = MovingActor.SOUTH;
		}
		else if(directionString.equals("south west"))
		{
			direction = MovingActor.SOUTH_WEST;
		}
		else if(directionString.equals("west"))
		{
			direction = MovingActor.WEST;
		}
		else if(directionString.equals("north west"))
		{
			direction = MovingActor.NORTH_WEST;
		}
		return direction;
	}
    
    private void init(int Direction)
    {
    	this.Direction = Direction;

    	String imageFile;    	
    	switch (Direction)
		{
			case MovingActor.NORTH:
				imageFile = "graphics/hint/northHint.png";
			break;
			case MovingActor.NORTH_EAST:
				imageFile = "graphics/hint/northEastHint.png";
			break;
			case MovingActor.EAST:
				imageFile = "graphics/hint/eastHint.png";
			break;
			case MovingActor.SOUTH_EAST:
				imageFile = "graphics/hint/southEastHint.png";
			break;
			case MovingActor.SOUTH:
				imageFile = "graphics/hint/southHint.png";
			break;
			case MovingActor.SOUTH_WEST:
				imageFile = "graphics/hint/southWestHint.png";
			break;
			case MovingActor.WEST:
				imageFile = "graphics/hint/westHint.png";
			break;
			case MovingActor.NORTH_WEST:
				imageFile = "graphics/hint/northWestHint.png";
			break;

			default :
				imageFile = "graphics/hint/northHint.png";
			break;
		}
		image = Cache.getImage(imageFile);
		width = image.getWidth(null);
		height = image.getHeight(null);
    	hitArea = new Rectangle(x, y, width, height);
    }
    
    public Element toXml(Document xmlDoc)
	{
		return addToElement(xmlDoc, xmlDoc.createElement("DirectionHint"));
	}
    
    protected Element addToElement(Document xmlDoc, Element element)
    {
    	super.addToElement(xmlDoc, element);
    	
		element.setAttribute("direction",directionToString());	
    	return element;
    }
    
    private String directionToString()
    {
    	String direction = null;
    	
		switch(Direction) //This case statement can be optmised more
        {
            case MovingActor.NORTH:
            	direction = "north";
            	break;
            case MovingActor.NORTH_EAST:
            	direction = "north east";
            	break;
            case MovingActor.EAST:
            	direction = "east";
            	break;
            case MovingActor.SOUTH_EAST:
            	direction = "south east";
            	break;
            case MovingActor.SOUTH:
            	direction = "south";
            	break;
            case MovingActor.SOUTH_WEST:
            	direction = "south west";
            	break;
            case MovingActor.WEST:
            	direction = "west";
            	break;
            case MovingActor.NORTH_WEST:
            	direction = "north west";
            	break;
        }
        return direction;
    }
    
    /**
     * Retuns the direction in which a MovingActor should travel
     * once it has collided with this DirectionHint
     */
    public int getDirection()
    {
    	return Direction;
    }
}
