package keenGame.sprite.monster;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import keenGame.Cache;
import keenGame.Level;
import keenGame.XmlElement;
import keenGame.sprite.Monster;
import keenGame.sprite.hint.LeftHint;
import keenGame.sprite.hint.RightHint;

/**
 * A BirdMonster, is a giant bird, that runs backwards, and forwards,
 * randomly, stopping when it hits either a LeftHint, or a RightHint
 * 
 * @see keenGame.sprite.hint.LeftHint
 * @see keenGame.sprite.hint.RightHint
 */
public class BirdMonster extends Monster
{
	private static final int FRAME_INTERVAL = 2;
	private static final int CHANGE_RANGE = 100;
	private static final int CHANGE_SHIFT = 100;
	
	private static final int STARS_X_OFFSET = 12;
	private static final int STARS_Y_OFFSET = -5;



    private int distance = 3;
    private int frame = 0;
    private int changeFrameCount = FRAME_INTERVAL;    
    private int changeTime;
    
    //Animations
    private Image [] leftAnim = new Image[4];
    private Image [] rightAnim = new Image[4];
    private Image stun;

	/**
     * Constructs an BirdMonster.  
     * 
     * @param layer - The layer that this BirdMonster should exist in
     * @param hitLayers - An array of strings containing the layers which contain
     * 	Sprites with which this BirdMonster can collide
     * @param xValue - The x location of the BirdMonster
     * @param yValue - The y location of the BirdMonster
	 * @param myWorld - the Level in which the MovingActor exists
     * 
     * @see keenGame.sprite.Sprite#hitArea
     * @see keenGame.sprite.Actor#Actor(String, String, int, int)
     */    
    public BirdMonster(String layer, String [] hitLayers, Level myWorld,
    					int xValue, int yValue)    
    {
        super(layer, hitLayers,myWorld,xValue,yValue);
        init();
    }
    
    public BirdMonster(XmlElement spriteElement, Level myLevel)
    {
    	super(spriteElement, myLevel);
    	init();
    }
    
    private void init()
    {
        //Load up the images
        
        //Left1
        leftAnim[0] = Cache.getImage("graphics/BirdMonster/birdLeft1.png");
        width = leftAnim[0].getWidth(null);
        height = leftAnim[0].getHeight(null);
        hitArea = new Rectangle(x, y, width, height);

        //Left2
        leftAnim[1] = Cache.getImage("graphics/BirdMonster/birdLeft2.png");

        //Left3
        leftAnim[2] = Cache.getImage("graphics/BirdMonster/birdLeft3.png");

        //Left4
        leftAnim[3] = Cache.getImage("graphics/BirdMonster/birdLeft4.png");


        //Right1
        rightAnim[0] = Cache.getImage("graphics/BirdMonster/birdRight1.png");

        //right2
        rightAnim[1] = Cache.getImage("graphics/BirdMonster/birdRight2.png");

        //right3
        rightAnim[2] = Cache.getImage("graphics/BirdMonster/birdRight3.png");

        //Right4
        rightAnim[3] = Cache.getImage("graphics/BirdMonster/birdRight4.png");
        
        //Stun
        stun = Cache.getImage("graphics/BirdMonster/birdStun.png");
        
        State = Cache.rNumbers.nextInt(2)+WALKING_LEFT;
		//Set the correct image
		if(State == WALKING_LEFT)
		{
			image = leftAnim[0];
		}
		else
		{
			image = rightAnim[0];
		}

    }
    
    public Element toXml(Document xmlDoc)
	{
		return addToElement(xmlDoc, xmlDoc.createElement("BirdMonster"));
	}
    
    /**
     * @see keenGame.sprite.Actor#update()
     */
    public void update()
    {
    	if(State != STUNED)
    	{
		    //Move
		    walk(State, distance);
		    
		    if(changeTime == 0)
		    {
		    	if(State == WALKING_LEFT)
		    	{
		    		State = WALKING_RIGHT;
		    	}
		    	else
		    	{
		    		State = WALKING_LEFT;
		    	}
   	    		frame = 0;
   	    		changeTime = Cache.rNumbers.nextInt(CHANGE_RANGE) + CHANGE_SHIFT;
		    }
		    else
		    {
		    	changeTime--;
		    }
		    	
    	}
		    //update the timers
		    nextUpdateTime = System.currentTimeMillis() + UPDATE_PERIOD;
		
    }
    
    /**
     * @see keenGame.sprite.Actor#setFrame()
     */
    public void setFrame()
    {
    	//Set to the correct animation sequence
    	if(changeFrameCount == 0)
	    {
	    	if(State == WALKING_LEFT)
	    	{
	    		if(frame == leftAnim.length-1)
	    		{
	    			frame = 0;
	    		}
	    		else
	    		{
	    			frame++;
	    		}
	    		image = leftAnim[frame];
	    	}
	    	else if(State == WALKING_RIGHT)
	    	{
	    	    if(frame == rightAnim.length-1)
	    		{
	    			frame = 0;
	    		}
	    		else
	    		{
	    			frame++;
	    		}
	    		image = rightAnim[frame];
	    	}
	    	else
	    	{
	    		image = stun;
	    	}
	    	changeFrameCount = FRAME_INTERVAL;
    	}
    	else
    	{
    		changeFrameCount--;
    	}
    }
    
    /**
     * Causes the BirdMonster to walk to the left
     */    
    public boolean hasHit(LeftHint hitSprite)
    {
    	if(State == WALKING_RIGHT)
    	{
    		State = WALKING_LEFT;
    		frame = 0;
    	   	return true;
    	}
    	
    	return false;
    }

    /**
     * Causes the BirdMonster to walk to the left
     */    
    public boolean hasHit(RightHint hitSprite)
    {
    	if(State == WALKING_LEFT)
    	{
    		State = WALKING_RIGHT;
    		frame = 0;
    	   	return true;
    	}
    	
    	return false;
    }
    
	/**
	 * If the BirdMonster is stuned, then there can be 
	 * no colision with it.  Otherwise similar to the
	 * method in Sprite
	 * 
	 * @see keenGame.sprite.Sprite#isCollision(Rectangle)
	 */
    public boolean isCollision(Rectangle movingSprite)
    {
    	if(State != STUNED)
    	{
		     return hitArea.intersects(movingSprite);
    	}
    	else
    	{
    		return false;
    	}
    }
    
    protected Point getStarsLocation()
    {
    	Point location = new Point();
    	
    	location.x = x + STARS_X_OFFSET;
    	location.y = y + STARS_Y_OFFSET;
    	
    	return location;
    }
}