package keenGame.sprite.tile;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.awt.Rectangle;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import keenGame.Cache;
import keenGame.XmlElement;
import keenGame.sprite.Sprite;

/**
 * AutoRectangleTiles can be used easily to create walls, floors, and 
 * other kinds of secnery.  A collison box the size of the graphic
 * associated with them will be automatically generated, so they can 
 * be used with just about any graphic, without any further work
 */
public class AutoRectangleTile extends Sprite
{
	
	private boolean canHit;
	private String graphic;

    /**
     * Constructs a AutoRectangleTile.  The hitArea will automatically be set
     * to the shape of the image.
     * 
     * @param imageLocation - The location of an image (normally .png format) that
     * 		is used for this AutoRectangleTile
     * @param layer - The layer that this AutoRectangleTile should exist in
     * @param xValue - The x location of the AutoRectangleTile
     * @param yValue - The y location of the AutoRectangleTile
     * @param canHit - Toggles if the AutoRectangleTile can be colided with or not
     * 
     * @see keenGame.sprite.Sprite#hitArea
     */	
    
    public AutoRectangleTile(String imageLocation, String layer, 
    						int xValue, int yValue, boolean canHit)
    {
    	super(layer,xValue,yValue);
    	init(imageLocation, canHit);
    }
    
    public AutoRectangleTile(XmlElement spriteElement)
    {
    	super(spriteElement);
    	
    	//boolean newCanHit = Boolean.getBoolean(spriteElement.getAttribute("canHit"));
    	
    	boolean newCanHit = false;
    	if(spriteElement.getAttribute("canHit").equals("true"))
		{
			newCanHit = true;
		}
    	
    	String newGraphic = spriteElement.getChild("graphic").getText();
    	
    	init(newGraphic, newCanHit);
    }
    
    private void init(String imageLocation, boolean canHit)
    {
    	image = Cache.getImage(imageLocation);
    	width = image.getWidth(null);
    	height = image.getHeight(null);
    	
    	this.canHit = canHit;
    	graphic = imageLocation;
    	
    	//Our floor just has a boring old square hitArea
    	hitArea = new Rectangle(x, y, width, height);
    }
    
	public Element toXml(Document xmlDoc)
	{
		return addToElement(xmlDoc, xmlDoc.createElement("AutoRectangleTile"));
	}
    
    protected Element addToElement(Document xmlDoc, Element element)
    {
    	super.addToElement(xmlDoc, element);
    	
    	//Graphic
    	Element graphicElement = xmlDoc.createElement("graphic");
		graphicElement.appendChild(xmlDoc.createTextNode(graphic));
		element.appendChild(graphicElement);
		
		//Attribute
		element.setAttribute("canHit", Boolean.toString(canHit));
    	
    	return element;
    }
    
	/**
	 * Returns true if hitArea.intersects(movingSprite) and
	 * canHit was set to true in the constructor
	 */
    public boolean isCollision(Rectangle movingSprite)
    {
     	return canHit && hitArea.intersects(movingSprite);
    }
}
