package keenGame.sprite.tile;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.awt.Point;
import java.awt.Polygon;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import keenGame.Cache;
import keenGame.XmlElement;

/**
 * A PolygonTile, is a ShapeTile, with a Polygon hitArea.  It should
 * be used  for odd shaped bits of scenery, walls, floors, etc, that keen
 * can grip hold of
 */
public class PolygonTile extends ShapeTile
{
	protected Point  rightPoint;
	protected Point  leftPoint;
	private String tileFile;
	
    /**
     * Constructs a PolygonTile.  
     * 
     * @param imageLocation - This should be the location of a PolygoneTile file
     * which defines the shape of thre polygone, as well as the imaged used.
     * @param layer - The layer that this PolygonTile should exist in
     * @param xValue - The x location of the PolygonTile
     * @param yValue - The y location of the PolygonTile
     * 
     * @see #hitArea
     */
    public PolygonTile(String imageLocation, String layer, int xValue, int yValue)
	{
    	super(layer, xValue,yValue);
    	init(imageLocation);
	}

    public PolygonTile(XmlElement spriteElement)
    {
    	super(spriteElement);
    	
    	String newFile = spriteElement.getChild("file").getText();
    	init(newFile);
    }
	
	private void init(String imageLocation)
	{
    	//load from disk
		PolygonTileFile file = Cache.getPolygonTile(imageLocation);
		tileFile = imageLocation;
		
		image = file.tileImage;
        width = image.getWidth(null);
        height = image.getHeight(null);		
        
		hitArea = new Polygon(file.polyX,file.polyY,file.polyX.length);
		((Polygon)hitArea).translate(x,y);
		
		if(file.rightPoint != null)
		{
			rightPoint = new Point(file.rightPoint);
			rightPoint.translate(x,y);
		}
		
		if(file.leftPoint != null)
		{
			leftPoint = new Point(file.leftPoint);
			leftPoint.translate(x,y);
		}
    }
    
	public Element toXml(Document xmlDoc)
	{
		return addToElement(xmlDoc, xmlDoc.createElement("PolygonTile"));
	}
    
    protected Element addToElement(Document xmlDoc, Element element)
    {
    	super.addToElement(xmlDoc, element);
    	
    	//File
    	Element graphicElement = xmlDoc.createElement("file");
		graphicElement.appendChild(xmlDoc.createTextNode(tileFile));
		element.appendChild(graphicElement);
		
    	return element;
    }

	/**
	 * @see keenGame.sprite.tile.ShapeTile#getRightGrip(int, int)
	 */
	public Point getRightGrip(int keenActorX, int keenActorY)
	{
		if(rightPoint != null)
		{
			if((keenActorX >= rightPoint.x ) && 
	    	  ((keenActorY > rightPoint.y) && (keenActorY < rightPoint.y + KEEN_GRAB_DISTANCE)))
	    	{
	    		return rightPoint;
	    	}
		}
   		return null;
	}

	/**
	 * @see keenGame.sprite.tile.ShapeTile#getLeftGrip(int, int)
	 */
	public Point getLeftGrip(int keenActorX, int keenActorY)
	{
		if(leftPoint != null)
		{
	    	if((keenActorX <= leftPoint.x) &&
		   	  ((keenActorY > leftPoint.y) && (keenActorY < leftPoint.y + KEEN_GRAB_DISTANCE)))
	    	{
	    		return leftPoint;
	    	}
		}
   		return null;
	}
}

