package keenGame.sprite.tile;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.awt.Image;
import java.awt.Rectangle;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import keenGame.Cache;
import keenGame.XmlElement;
import keenGame.sprite.Sprite;
import keenGame.sprite.SwitchListener;

/**
 * Switches can be activated, or deactivated by keen.  They
 * can be used to controll a number of things. To listen to
 * switches an object needs to implement the SwitchListener
 * interface.
 */
public class Switch extends Sprite
{
	private static final int MAX_LISTENERS = 10;

	private Image onImage;
	private Image offImage;

	private boolean on = true;
	private int currentListeners = 0;
	private SwitchListener [] listeners = new SwitchListener[MAX_LISTENERS];

    /**
     * Constructs a Switch.  
     * 
     * @param imageLocation - This should be the location of a RectangleTile file
     * which defines the shape of thre rectangle, as well as the imaged used.
     * @param layer - The layer that this RectangleTile should exist in
     * @param xValue - The x location of the RectangleTile
     * @param yValue - The y location of the RectangleTile
     * @param on - weather the switch should be activated, or deactivated
     * 		when it is constructed.
     */
    public Switch(String layer, int xValue, int yValue, boolean on)
	{
    	super(layer, xValue,yValue);
    	init(on);
	}

    public Switch(XmlElement spriteElement)
    {
    	super(spriteElement);
    	
    	boolean newIsOn = false;
    	if(spriteElement.getAttribute("isOn").equals("true"))
		{
			newIsOn = true;
		}
    	
    	init(newIsOn);
    }
	
	private void init(boolean on)
	{
    	onImage = Cache.getImage("graphics/switch/pushButtonOn.png");
    	offImage = Cache.getImage("graphics/switch/pushButtonOff.png");
    	
    	this.on = on;
    	if(on)
    	{
    		image = onImage;
    	}
    	else
    	{
    		image = offImage;
    	}
    	width = image.getWidth(null);
    	height = image.getHeight(null);
    	hitArea = new Rectangle(x, y, width, height);
    }
    
    public Element toXml(Document xmlDoc)
	{
		return addToElement(xmlDoc, xmlDoc.createElement("Switch"));
	}
    
    protected Element addToElement(Document xmlDoc, Element element)
    {
    	super.addToElement(xmlDoc, element);
		
		//Attribute
		element.setAttribute("isOn", Boolean.toString(on));
    	
    	return element;
    }
    
    /**
     * Toggles the state of the switch. If the switched is active, it will
     * deactivate.  If it is not active, it will activate.  It will
     * notify all its listeners of its state
     */
    public void toggle()
    {
   		on = !on;
    	if(on)
    	{
    		image = onImage;
    	}
    	else
    	{
    		image = offImage;
    	}
    	
    	//notify our listeners
    	for (int i = 0; i < currentListeners; i++)
		{
			listeners[i].SwitchToggled(this);	
		}
    }
 
	/**
	 * Returns the state of the switch
	 * 
	 * @return true if the switch is active, false if it is not
	 */ 
    public boolean isOn()
    {
    	return on;
    }

	/**
	 * Notifies the switch, that the specifed SwitchListener wishes
	 * to know when the state of the switch is changed. All listeners
	 * will be notified when the toggle method of this switch is called
	 * 
	 * @param newListener - the SwitchListener that wishes to be notifed
	 * 			when this switch is toggled
	 */
    public void addSwitchListener(SwitchListener newListener)
    {
    	if(currentListeners != MAX_LISTENERS)
    	{
    		listeners[currentListeners] = newListener;
    		newListener.SwitchToggled(this);
    	}
    	currentListeners++;
    }
}
