package keenGame.test;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.util.Iterator;
import java.util.Vector;

import junit.framework.TestCase;
import keenGame.ArrayMap;
import keenGame.sprite.MovementEvent;
import keenGame.sprite.Sprite;

public class ArrayMapTest extends TestCase
{
	private ArrayMap testMap;
	private TestSprite sprite1, sprite2, sprite3;
	/**
	 * Constructor for ArrayMapTest.
	 * @param arg0
	 */
	public ArrayMapTest(String arg0)
	{
		super(arg0);
	}

	/**
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception
	{
		super.setUp();
		
		String [] layers = {"layer1","layer2","layer3"};
		testMap = new ArrayMap(500, 500, layers, 10, 3);

		sprite1 = new TestSprite(10,10,20,20,"layer1");
		sprite2 = new TestSprite(40,40,20,20,"layer2");
		sprite3 = new TestSprite(60,40,20,20,"layer3");

		testMap.add(sprite1);
		testMap.add(sprite2);
		testMap.add(sprite3);
	}
	
	public void testAddLayer()
	{
		testMap.addLayer("layer4");
		
		String [] layers = testMap.getLayers();
		assertTrue(layers.length == 4);
		assertTrue(layers[3].equals("layer4"));
		
		TestSprite sprite4 = new TestSprite(60,40,20,20,"layer3");
		testMap.add(sprite4);
		assertTrue(testMap.contains(sprite4));
	}
	
	public void testRemoveLayer()
	{
		testMap.removeLayer("layer2");
		assertTrue(testMap.getLayers().length == 2);
		
		assertTrue(testMap.contains(sprite1));
		assertTrue(!testMap.contains(sprite2));	
		assertTrue(testMap.contains(sprite3));		
	}
	
	public void testRenameLayer()
	{
		testMap.renameLayer("layer2","blah");
		assertTrue(testMap.getLayers().length == 3);
		assertTrue(testMap.getLayers()[1].equals("blah"));
		
		assertTrue(testMap.contains(sprite1));
		assertTrue(testMap.contains(sprite2));	
		assertTrue(testMap.contains(sprite3));
		
		assertTrue(sprite2.getLayer().equals("blah"));
	}
	
	public void testSwapLayers()
	{
		testMap.swapLayers("layer2","layer1");
		assertTrue(testMap.getLayers().length == 3);
		assertTrue(testMap.getLayers()[0].equals("layer2"));
		assertTrue(testMap.getLayers()[1].equals("layer1"));
		
		assertTrue(testMap.contains(sprite1));
		assertTrue(testMap.contains(sprite2));	
		assertTrue(testMap.contains(sprite3));
		
		//get Iterator
		Iterator it = testMap.iterator();
		
		//check they come out
		assertTrue(it.next() == sprite2);
		assertTrue(it.next() == sprite1);
	}

	public void testSize()
	{
		assertTrue(testMap.size() == 3);
		
		//Remove a few sprites
		testMap.remove(sprite2);
		assertTrue(testMap.size() == 2);
		
		testMap.remove(sprite1);
		assertTrue(testMap.size() == 1);

		testMap.remove(sprite3);
		assertTrue(testMap.size() == 0);
	}

	public void testClear()
	{
		//clear
		testMap.clear();		
		
		//Check the size
		assertTrue(testMap.size() == 0);
		
		//Check iterator gives us nothing
		assertTrue(!testMap.iterator().hasNext());
	}

	/*
	 * Test for void ArrayMap(int, int, String[], int, int)
	 */
	public void testGetLayers()
	{
		//Check it gives the proper layers
		String [] layers = testMap.getLayers();
		
		assertTrue(layers[0] == "layer1");
		assertTrue(layers[1] == "layer2");
		assertTrue(layers[2] == "layer3");
	}

	/*
	 * Test for boolean add(Object)
	 */
	public void testAddObject()
	{
		//check contains
		assertTrue(testMap.contains(sprite1));
		assertTrue(testMap.contains(sprite2));
		assertTrue(testMap.contains(sprite3));
		
		//Lets try with one that shouldnt work
		Sprite sprite4 = new TestSprite(10,10,20,20,"layerFoo");
		try
		{
			testMap.add(sprite4);
			fail("Allowed adding of sprite with non-existent layer");
		}
		catch(UnsupportedOperationException e)
		{}
	}

	/*
	 * Test for boolean remove(Object)
	 */
	public void testRemoveObject()
	{
		//Remove a few sprites
		testMap.remove(sprite2);
		assertTrue(!testMap.contains(sprite2));
		assertTrue(testMap.contains(sprite1));
		assertTrue(testMap.contains(sprite3));
		
		testMap.remove(sprite1);
		assertTrue(!testMap.contains(sprite1));
		assertTrue(testMap.contains(sprite3));

		testMap.remove(sprite3);
		assertTrue(!testMap.contains(sprite3));
	}
	
	public void testRemoveAll()
	{
		Vector toRemove = new Vector();
		toRemove.add(sprite1);
		toRemove.add(sprite2);
		
		testMap.removeAll(toRemove);
		
		assertTrue(!testMap.contains(sprite1));
		assertTrue(!testMap.contains(sprite2));
		assertTrue(testMap.contains(sprite3));
	}

	/*
	 * Test for Iterator iterator()
	 */
	public void testIterator()
	{
		//get Iterator
		Iterator it = testMap.iterator();
		
		//check they come out
		assertTrue(it.hasNext());
		assertTrue(it.next() == sprite1);

		assertTrue(it.hasNext());
		assertTrue(it.next() == sprite2);

		assertTrue(it.hasNext());
		assertTrue(it.next() == sprite3);
		
		assertTrue(!it.hasNext());
	}

	/*
	 * Test for Iterator iterator(String)
	 */
	public void testIteratorString()
	{
		//get Iterator
		Iterator it = testMap.iterator("layer2");

		//check one comes out
		assertTrue(it.hasNext());
		assertTrue(it.next() == sprite2);
		assertTrue(!it.hasNext());
	}

	/*
	 * Test for Iterator iterator(String[])
	 */
	public void testIteratorStringArray()
	{
		//get Iterator
		String [] layers = {"layer1", "layer3"};
		Iterator it = testMap.iterator(layers);

		//check two comes out
		assertTrue(it.hasNext());
		assertTrue(it.next() == sprite1);

		assertTrue(it.hasNext());
		assertTrue(it.next() == sprite3);
		assertTrue(!it.hasNext());
	}

	/*
	 * Test for Iterator iterator(int, int, int, int)
	 */
	public void testIteratorIIII()
	{
		//get Iterator
		Iterator it = testMap.iterator(0,0,20,20);
		
		//check one come out
		assertTrue(it.hasNext());
		assertTrue(it.next() == sprite1);
		assertTrue(!it.hasNext());
	}

	/*
	 * Test for Iterator iterator(int, int, int, int, String)
	 */
	public void testIteratorIIIIString()
	{
		//get Iterator
		Iterator it = testMap.iterator(0,0,20,20, "layer1");
		
		//check one come out
		assertTrue(it.hasNext());
		assertTrue(it.next() == sprite1);
		assertTrue(!it.hasNext());

		//Now try when we should get none
		it = testMap.iterator(0,0,20,20, "layer2");
		
		//check one come out
		assertTrue(!it.hasNext());
	}

	/*
	 * Test for Iterator iterator(int, int, int, int, String[])
	 */
	public void testIteratorIIIIStringArray()
	{
		//get Iterator
		String [] layers = {"layer1", "layer3"};
		Iterator it = testMap.iterator(0,0,20,20, layers);

		//check one comes out
		assertTrue(it.hasNext());
		assertTrue(it.next() == sprite1);
		assertTrue(!it.hasNext());
	}
	
	public void testBig()
	{
		String [] layers = {"layer1", "layer2", "layer3"};

		ArrayMap testLayer = new ArrayMap(500,500, layers);
		Iterator it = testLayer.iterator(0,0,500,500);
		
		while(it.hasNext())
		{
			it.next();
		}
	}


	public void testMovementPerformed()
	{
		//Move 100 right
		for(int distance=0; distance< 100; distance++)
		{
			//move it
			sprite1.right();
			
			//Throw in movement event
			testMap.movementPerformed(new MovementEvent(sprite1, 
														sprite1.getX()-1,
														sprite1.getY(),
														sprite1.getX(),
														sprite1.getY()));
		}
		
		//Check with iterator
		Iterator it = testMap.iterator(110,10,130,30);
		assertTrue(it.hasNext());
		assertTrue(it.next() == sprite1);
		assertTrue(!it.hasNext());
		
		
		//Move 100 down
		for(int distance=0; distance< 100; distance++)
		{
			//move it
			sprite1.down();
			
			//Throw in movement event
			testMap.movementPerformed(new MovementEvent(sprite1, 
														sprite1.getX(),
														sprite1.getY()-1,
														sprite1.getX(),
														sprite1.getY()));
		}
		
		//Check with iterator
		it = testMap.iterator(110,110,130,130);
		assertTrue(it.hasNext());
		assertTrue(it.next() == sprite1);
		assertTrue(!it.hasNext());
	}
}
