package keenGame.sprite.bullet;

/*
 * Copyright (C) Luke Sleeman 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.awt.Image;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import keenGame.Cache;
import keenGame.Level;
import keenGame.sprite.Bullet;
//import keenGame.sprite.keenActor.KeenActor;
import keenGame.sprite.tile.ThinFloor;

/**
 * A Blast is a shot fired by Commander Keens gun.  It will explode
 * on the first sprite it hits, disabling them if they are a Monster
 */
public class Blast extends Bullet
{
    //Animations
    private Image [] flight = new Image[4];
    private Image [] explode = new Image[2];
    
    private int direction;
    private int distance = 13;
	private static final int FRAME_INTERVAL = 2;

	/**
     * Constructs an Blast.  The blast will immediately fly off in the
     * direction specified in the constructor.
     * 
     * @param imageLocation - The location of an image (normally .png format) that
     * 		is used for this Blast
     * @param layer - The layer that this Blast should exist in
     * @param hitLayers - An array of strings containing the layers which contain
     * 	Sprites with which this Blast can collide
     * @param xValue - The x location of the Blast
     * @param yValue - The y location of the Blast
     * @param direction - The direction that the Blast should head in.  One
     * 	of the constants from MovingActor
     * 
     * @see keenGame.sprite.Sprite#hitArea
     * @see keenGame.sprite.Actor#Actor(String, String, int, int)
     * @see keenGame.sprite.MovingActor
     */
    public Blast(String layer, String [] hitLayers, Level myWorld,
    					int xValue, int yValue, int direction)    
    {
        super(layer, hitLayers, myWorld, xValue,yValue);
        
        //Load up the images
		//Flight
        flight[0] = Cache.getImage("graphics/blast/blast0.png");
        flight[1] = Cache.getImage("graphics/blast/blast1.png");
        flight[2] = Cache.getImage("graphics/blast/blast2.png");
        flight[3] = Cache.getImage("graphics/blast/blast3.png");

		//Die
        explode[0] = Cache.getImage("graphics/blast/blastDie0.png");
        explode[1] = Cache.getImage("graphics/blast/blastDie1.png");
        
        State = FLYING;
        this.direction = direction;
        width = flight[0].getWidth(null);
        height = flight[0].getHeight(null);
        image = flight[0];
    }
    
	public Element toXml(Document xmlDoc)
	{
		throw new UnsupportedOperationException("Blasts cannot be saved");
	}
    
    /**
     * Moves the blast in the direction specified in the constructor
     * @see keenGame.sprite.Actor#update()
     */
    public void update()
    {
	    //Move
	    if(State == FLYING)
	    {
	    	directionMove(distance, direction);
	    }
	    //update the timers
	    nextUpdateTime = System.currentTimeMillis() + UPDATE_PERIOD;
    }

    /**
     * @see keenGame.sprite.Actor#setFrame()
     */
    public void setFrame()
    {
    	//Set to the correct animation sequence
    	if(changeFrameCount == 0)
	    {
   	    	changeFrameCount = FRAME_INTERVAL;
	    	if(State == FLYING)
	    	{
	    		if(frame == flight.length-1)
	    		{
	    			frame = 0;
	    		}
	    		else
	    		{
	    			frame++;
	    		}
	    		image = flight[frame];
	    	}
	    	if(State == EXPLODING)
	    	{
	    		if(frame == 0)
	    		{
		    		image = explode[frame];
		    		frame++;
		    		changeFrameCount = FRAME_INTERVAL*2;
	    		}
	    		else if(frame == 1)
	    		{
		    		image = explode[frame];
		    		frame++;
		    		changeFrameCount = FRAME_INTERVAL*2;
	    		}
	    		else if(frame == 2)
	    		{
	    			enabled = false;
	    		}
	    		else
	    		{
	    			System.err.println ("nope " + frame);
	    		}
	    	}
    	}
    	
    	else
    	{
    		changeFrameCount--;
    	}
    }
    
    /**
     * Allows the blast to go through ThinFloors, if it is heading upwards
     */
    public boolean hasHit(ThinFloor hitSprite)
    {
		if((State != EXPLODING) && (direction == SOUTH))
		{
    		State = EXPLODING;
    		changeFrameCount = 0;
    		frame = 0;
    		return true;
    	}
    	else
    	{
	    	return false;
    	}
    }

	/*    
    public boolean hasHit(KeenActor hitSprite)
    {
    	return false;
    }
    */
}
